@@ -23,7 +23,7 @@
  *
  * Original author: Eric S. Raymond (Fetchmail)
  * Heavily modified by Brendan Cully <brendan@kublai.com> (Mutt)
- * Modify the code for generic use by Timo Schulz <ts@winpt.org>
+ * Modify the code for generic use by Timo Schulz <twoaday@freakmail.de>
  */
 
 #ifdef HAVE_CONFIG_H
@@ -31,13 +31,18 @@
 #endif
 #include <stdio.h>
 #include <string.h>
+#include <assert.h>
 #include <sys/stat.h>
 
 #include "opencdk.h"
 #include "main.h"
 #include "filters.h"
 
-#define LF "\n" /* You need to change this for other OS's (W32: "\r\n") */
+#ifdef __MINGW32__
+# define LF "\r\n"
+#else
+# define LF "\n"
+#endif
 
 #define CRCINIT 0xB704CE
 
@@ -110,7 +115,8 @@ static const char *valid_headers[] =
 static char b64chars[64] =
 "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
 
-static int index64[128] = {
+static int index64[128] =
+{
   -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
   -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
   -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 62, -1, -1, -1, 63,
@@ -277,8 +283,8 @@ update_crc (unsigned crc, const byte *buf, size_t count)
   if (!crc)
     crc = CRCINIT;
   for (j = 0; j < count; j++)
-      crc = (crc << 8) ^ crc_table[0xff & ((crc >> 16) ^ buf[j])];
-  crc &= 0x00ffffff;
+    crc = (crc << 8) ^ crc_table[0xff & ((crc >> 16) ^ buf[j])];
+  crc &= 0xffffff;
   return crc;
 }
 
@@ -288,7 +294,7 @@ armor_encode (void * opaque, FILE * in, FILE * out)
 {
   armor_filter_s * afx = opaque;
   struct stat statbuf;
-  char crcbuf[5], buf[4096], raw[1023];
+  char crcbuf[5], buf[4096], raw[1024];
   size_t nread = 0;
   const char * lf;
   int rc = 0, i = 0, pos = 0;
@@ -379,7 +385,7 @@ armor_decode (void * opaque, FILE *in, FILE *out)
 {
   armor_filter_s * afx = opaque;
   const char *s = NULL;
-  char buf[128];
+  char buf[127];
   byte raw[128];
   u32 crc2 = 0;
   size_t nread = 0;
@@ -450,7 +456,7 @@ armor_decode (void * opaque, FILE *in, FILE *out)
       buf[strlen (buf) - strlen (LF)] = '\0';
       if (buf[0] == '=' && strlen (s) == 5) /* CRC */
 	{
-	  byte crcbuf[3];
+	  byte crcbuf[4];
 	  memset (crcbuf, 0, sizeof crcbuf);
 	  base64_decode (crcbuf, buf + 1);
 	  crc2 = crcbuf[0] << 16 | crcbuf[1] << 8 | crcbuf[2];
@@ -519,8 +525,6 @@ cdk_file_armor (CDK_HD hd, const char *file, const char *output)
     }
 
   cdk_stream_set_armor_flag (out, CDK_ARMOR_MESSAGE);
-  if (hd && hd->opt.compress)
-    cdk_stream_set_compress_flag (out, hd->compress_algo);
   cdk_stream_set_literal_flag (out, 0, file);
 
   rc = cdk_stream_kick_off (inp, out);
@@ -531,7 +535,7 @@ cdk_file_armor (CDK_HD hd, const char *file, const char *output)
   return rc;
 }
 
-
+#include "stream.h"
 /**
  * cdk_file_dearmor: Remove ASCII armor from a file.
  * @file: Name of the file to unprotect.
@@ -563,7 +567,7 @@ cdk_file_dearmor (const char *file, const char *output)
       cdk_stream_set_literal_flag (inp, 0, NULL);
       cdk_stream_set_armor_flag (inp, 0);
     }
-
+  
   rc = cdk_stream_kick_off (inp, out);
 
   cdk_stream_close (inp);
