@@ -34,7 +34,7 @@ _cdk_cipher_test_algo (int algo)
 {
   if (algo < 0 || algo > 110)
     return CDK_Inv_Algo;
-  return gcry_cipher_test_algo (algo);
+  return gcry_cipher_test_algo (algo)? CDK_Inv_Algo : 0;
 }
 
 
@@ -42,8 +42,8 @@ int
 _cdk_md_test_algo (int algo)
 {
   if (algo < 0 || algo > 110)
-    return GCRYERR_INV_MD_ALGO;
-  return gcry_md_test_algo (algo);
+    return CDK_Inv_Algo;
+  return gcry_md_test_algo (algo) ? CDK_Inv_Algo : 0;
 }
 
 
@@ -71,7 +71,7 @@ hash_filter (void * opaque, FILE * in, FILE * out)
         break;
       gcry_md_write (mfx->md, buf, nread);
     }  
-
+  wipemem (buf, sizeof buf);
   return 0;
 }
 
@@ -138,6 +138,7 @@ write_header (cipher_filter_s * cfx, CDK_STREAM out)
         rc = CDK_Weak_Key;
       else
         rc = CDK_Gcry_Error;
+      gcry_md_close (cfx->mdc_hash);
       return rc;
     }
   gcry_cipher_setiv (cfx->cipher_hd, NULL, 0);
@@ -176,7 +177,7 @@ cipher_encode2 (void * opaque, FILE * in, FILE * out)
         }
       fwrite (buf, 1, nread, out);
     }
-  
+  wipemem (buf, sizeof buf);
   if (!rc && cfx->mdc_hash)
     {
       if (dlen != 20)
@@ -276,6 +277,7 @@ cipher_decode2 (void * opaque, FILE * in, FILE * out)
         gcry_md_write (cfx->mdc_hash, buf, nread);
       fwrite (buf, 1, nread, out);
     }
+  wipemem (buf, sizeof buf);
   return rc;
 }
 
