@@ -63,44 +63,33 @@ static int
 decompress_data (compress_filter_s * zfx, z_stream * zs,
                  FILE * in, size_t * ret_len)
 {
-  int refill = !zs->avail_in;
-  int count, n, nread;
+  int nread = 0, nold;
   int rc = 0, zrc = 0;
-  
-  do {
-    if (zs->avail_in < zfx->inbufsize && refill)
-      {
-        n = zs->avail_in;
-        if (!n)
-          zs->next_in = zfx->inbuf;
-        count = zfx->inbufsize - n;
-        nread = fread (zfx->inbuf + n, 1, count, in);
-        n += nread;
 
-        /* If we use the undocumented feature to suppress the zlib header,
-           we have to give inflate an extra dummy byte to read */
-        if (nread < count && zfx->algo == 1)
-          {
-            *(zfx->inbuf + n) = 0xFF; /* is it really needed ? */
-            zfx->algo1hack = 1;
-            n++;
-          }
-        zs->avail_in = n;
-      }
-    refill = 1;
-    zrc = inflate (zs, Z_PARTIAL_FLUSH);
-    if (zrc == Z_STREAM_END)
-      rc = EOF; /* eof */
-    else if (zrc != Z_OK && zrc != Z_BUF_ERROR)
-      {
-        rc = CDK_Zlib_Error;
+  while (zs->avail_out != 0)
+    {
+      if (!zs->avail_in)
+        {
+          nread = fread (zfx->inbuf, 1, zfx->inbufsize, in);
+          zs->next_in = zfx->inbuf;
+          zs->avail_in = nread;
+        }
+      nold = zs->avail_out;
+      zrc = inflate (zs, Z_SYNC_FLUSH);
+      if (zrc != Z_OK && zrc != Z_STREAM_END)
+        {
+          rc = CDK_Zlib_Error;
+          break;
+        }
+      *ret_len = zfx->outbufsize - zs->avail_out;
+      if (nold == zs->avail_out)
         break;
-      }
-  }
-  while (zs->avail_out && zrc != Z_STREAM_END && zrc != Z_BUF_ERROR);
-  *ret_len = zfx->outbufsize - zs->avail_out;
-  if (rc == EOF)
-    rc = 0;
+      if (zrc == Z_STREAM_END)
+        {
+          rc = EOF; /* eof */
+          break;
+        }
+    }
   return rc;
 }
 
@@ -124,23 +113,22 @@ compress_decode (void * opaque, FILE * in, FILE * out)
   if (zrc != Z_OK)
     return CDK_Zlib_Error;
 
+  zfx->outbufsize = 8192;
   zfx->inbufsize = 2048;
   memset (zfx->inbuf, 0, sizeof zfx->inbuf);
   zs->avail_in = 0;  
 
-  zs->next_out = zfx->outbuf;
-  zs->avail_out = 8192;
-  zfx->outbufsize = 8192;
-
-  while (!feof (in))
+  while (rc != -1)
     {
+      zs->next_out = zfx->outbuf;
+      zs->avail_out = 8192;
       rc = decompress_data (zfx, zs, in, &nbytes);
-      if (rc)
-        break;
       fwrite (zfx->outbuf, 1, nbytes, out);
     }
   inflateEnd (zs);
   cdk_free (zs);
+  if (rc == CDK_EOF)
+    rc = 0;
   return rc;
 }
   
