@@ -35,6 +35,14 @@ struct cdk_verify_result_s
   char * notation;
 };
 
+struct cdk_s2k_hd_s
+{
+  int mode;
+  unsigned char hash_algo;
+  unsigned char salt[8];
+  unsigned long count;
+};
+
 struct cdk_handle_s 
 {
   int cipher_algo;
@@ -45,18 +53,24 @@ struct cdk_handle_s
     int mode;
     int digest_algo;
     int cipher_algo;
-  } s2k;
+  } _s2k;
   struct 
   {
     unsigned armor:1;
     unsigned textmode:1;
     unsigned compress:1;
     unsigned mdc:1;
+    unsigned compat:1;
   } opt;
   struct 
   {
     Verify_Result verify;
   } result;
+
+  CDK_DEK dek;
+  CDK_S2K_HD s2k;
+  void (*callback) (void * opaque, int type, const char * s);
+  void *callback_value;
 };
 
 struct cdk_prefitem_s
