@@ -22,6 +22,8 @@
 # include <config.h>
 #endif
 #include <stdio.h>
+#include <stdarg.h>
+#include <malloc.h>
 #include <assert.h>
 
 #include "opencdk.h"
@@ -53,52 +55,57 @@ struct mainproc_ctx_s
 typedef struct mainproc_ctx_s *CTX;
 
 
+static void
+write_marker_packet (CDK_STREAM out)
+{
+  byte buf[5];
+
+  buf[0] = (0x80 | (10<<2));
+  buf[1] = 3;
+  buf[2] = 0x50;
+  buf[3] = 0x47;
+  buf[4] = 0x50;
+  cdk_stream_write (out, buf, 5);
+}
+
+
 static int
-sym_encrypt_file (CDK_HD hd, const char *file, const char *output)
+sym_stream_encrypt (CDK_HD hd, CDK_STREAM inp, CDK_STREAM out)
 {
-  CDK_STREAM inp = NULL, out = NULL;
-  CDK_S2K_HD *s2k = NULL;
   CDK_PACKET pkt;
-  CDK_DEK dek = NULL;
-  cdkPKT_symkey_enc *enc;
-  char *pw = NULL;
+  cdkPKT_symkey_enc * enc;
+  char * pw = NULL;
   int rc = 0;
 
-  if (!hd || !file || !output)
+  if (!hd || !inp || !out)
     return CDK_Inv_Value;
 
-  rc = cdk_stream_open (file, &inp);
-  if (rc)
-    return rc;
-  
-  rc = cdk_stream_new (output, &out);
-  if (rc)
-    {
-      cdk_stream_close (inp);
-      return rc;
-    }
-  
   pw = _cdk_passphrase_get ("Enter Passphrase: ");
   if (!pw)
     goto fail;
 
-  s2k = cdk_calloc (1, sizeof *s2k);
-  if (!s2k)
+  cdk_free (hd->s2k);
+  hd->s2k = cdk_calloc (1, sizeof *hd->s2k);
+  if (!hd->s2k)
     {
       rc = CDK_Out_Of_Core;
       goto fail;
     }
-  s2k->mode = hd->s2k.mode;
-  s2k->hash_algo = hd->s2k.digest_algo;
-  
-  dek = cdk_passphrase_to_dek (hd->cipher_algo, s2k, 2, pw);
-  if (!dek || !dek->keylen)
+  hd->s2k->mode = hd->_s2k.mode;
+  hd->s2k->hash_algo = hd->_s2k.digest_algo;
+
+  cdk_free (hd->dek);
+  hd->dek = cdk_passphrase_to_dek (hd->cipher_algo, hd->s2k, 2, pw);
+  if (!hd->dek || !hd->dek->keylen)
     {
       rc = CDK_General_Error;
       goto fail;
     }  
 
   cdk_stream_set_cache (out, 1);
+  if (hd->opt.compat)
+    write_marker_packet (out);
+      
   enc = cdk_calloc (1, sizeof *enc);
   if (!enc)
     {
@@ -106,8 +113,8 @@ sym_encrypt_file (CDK_HD hd, const char *file, const char *output)
       goto fail;
     }
   enc->version = 4;
-  enc->cipher_algo = dek->algo;
-  enc->s2k = *s2k;
+  enc->cipher_algo = hd->dek->algo;
+  enc->s2k = hd->s2k;
   init_packet (&pkt);
   pkt.pkttype = CDK_PKT_SYMKEY_ENC;
   pkt.pkt.symkey_enc = enc;
@@ -117,47 +124,93 @@ sym_encrypt_file (CDK_HD hd, const char *file, const char *output)
     goto fail;
   cdk_stream_set_cache (out, 0);
 
-  cdk_stream_set_cipher_flag (out, dek, hd->opt.mdc);
-  cdk_stream_set_compress_flag (out, hd->compress_algo);
-  cdk_stream_set_literal_flag (out, 0, file);
+  if (hd->opt.armor)
+    cdk_stream_set_armor_flag (out, 0);
+  cdk_stream_set_cipher_flag (out, hd->dek, hd->opt.mdc);
+  if (hd->opt.compress)
+    cdk_stream_set_compress_flag (out, hd->compress_algo);
+  cdk_stream_set_literal_flag (out, 0, _cdk_stream_get_fname (inp));
 
   rc = cdk_stream_kick_off (inp, out);
 
 fail:
-  cdk_stream_close (inp);
-  cdk_stream_close (out);
-  cdk_free (s2k);
-  cdk_free (dek);
   _cdk_passphrase_free (pw);
   return rc;
 }
 
 
 /**
- * cdk_file_encrypt: Encrypt a file.
+ * cdk_stream_encrypt: Encrypt a stream.
  * @hd: Handle
  * @remusr: List of recipients
- * @file: Name of the file to encrypt
- * @output: Output filename
+ * @inp: Input stream handle
+ * @out: Output stream handle
  *
  * If remusr is NULL, then symmetric encryption is used. Via the
  * handle the caller can set or unset multiple options.
  */
 int
-cdk_file_encrypt (CDK_HD hd, CDK_STRLIST remusr,
-                  const char *file, const char *output)
+cdk_stream_encrypt (CDK_HD hd, CDK_STRLIST remusr,
+                    CDK_STREAM inp, CDK_STREAM out)
 {
   CDK_KEYLIST pkl = NULL;
-  CDK_STREAM inp = NULL, out = NULL;
-  CDK_DEK dek = NULL;
+  int cipher_algo;
   int rc = 0;
 
-  if (!file || !output)
+  if (!hd || !inp || !out)
     return CDK_Inv_Value;
 
   if (!remusr)
-    return sym_encrypt_file (hd, file, output);
+    return sym_stream_encrypt (hd, inp, out);
 
+  rc = cdk_pklist_build (remusr, &pkl, GCRY_PK_USAGE_ENCR);
+  if (rc)
+    return rc;
+
+  cipher_algo = cdk_pklist_select_algo (pkl);
+  cdk_free (hd->dek);
+  rc = cdk_dek_new (&hd->dek, cipher_algo);
+  if (rc)
+    {
+      cdk_pklist_release (pkl);
+      return rc;
+    }
+  
+  hd->dek->use_mdc = cdk_pklist_use_mdc (pkl);
+
+  cdk_stream_set_cache (out, 1);
+  if (hd->opt.compat)
+    write_marker_packet (out);
+  
+  rc = cdk_pklist_encrypt (pkl, hd->dek, out);
+  if (rc)
+    {
+      cdk_pklist_release (pkl);
+      return rc;
+    }
+  cdk_stream_set_cache (out, 0);
+
+  if (hd->opt.armor)
+    cdk_stream_set_armor_flag (out, 0);
+  cdk_stream_set_cipher_flag (out, hd->dek, 0);
+  if (hd->opt.compress)
+    cdk_stream_set_compress_flag (out, hd->compress_algo);
+  cdk_stream_set_literal_flag (out, 0, _cdk_stream_get_fname (inp));
+
+  rc = cdk_stream_kick_off (inp, out);
+
+  cdk_pklist_release (pkl);
+  return rc;
+}
+
+
+int
+cdk_file_encrypt (CDK_HD hd, CDK_STRLIST remusr,
+                  const char * file, const char * output)
+{
+  CDK_STREAM inp, out;
+  int rc;
+  
   rc = cdk_stream_open (file, &inp);
   if (rc)
     return rc;
@@ -166,37 +219,14 @@ cdk_file_encrypt (CDK_HD hd, CDK_STRLIST remusr,
   if (rc)
     {
       cdk_stream_close (inp);
-      return rc;
+      return rc; 
     }
 
-  rc = cdk_pklist_build (remusr, &pkl, GCRY_PK_USAGE_ENCR);
-  if (rc)
-    goto fail;
+  rc = cdk_stream_encrypt (hd, remusr, inp, out);
 
-  rc = cdk_dek_new (&dek, 0);
-  if (rc)
-    goto fail;
-  dek->algo = cdk_pklist_select_algo (pkl);
-  dek->use_mdc = cdk_pklist_use_mdc (pkl);
-
-  cdk_stream_set_cache (out, 1);
-  rc = cdk_pklist_encrypt (pkl, dek, out);
-  if (rc)
-    goto fail;
-  cdk_stream_set_cache (out, 0);
-
-  cdk_stream_set_cipher_flag (out, dek, 0);
-  cdk_stream_set_compress_flag (out, hd->compress_algo);
-  cdk_stream_set_literal_flag (out, 0, file);
-
-  rc = cdk_stream_kick_off (inp, out);
-
-fail:
   cdk_stream_close (inp);
   cdk_stream_close (out);
-  cdk_free (dek);
-  cdk_pklist_release (pkl);
-  
+
   return rc;
 }
 
@@ -212,6 +242,40 @@ set_reset (CTX c, CDK_STREAM inp)
 }
 
 
+static void
+write_status (CDK_HD hd, int type, const char * fmt, ...)
+{
+  va_list arg_ptr;
+  char * buf;
+  int n;
+  
+  if (!hd->callback)
+    return;
+
+  va_start (arg_ptr, fmt);
+  n = vasprintf (&buf, fmt, arg_ptr);
+  buf[n] = '\0';
+  hd->callback (hd->callback_value, type, buf);
+  free (buf);
+  va_end (arg_ptr);
+}
+
+
+static int
+is_openpgp_ext (const char * file)
+{
+  int yes = 0;
+
+  if (strstr (file, ".asc")
+      || strstr (file, ".gpg")
+      || strstr (file, ".pgp")
+      || strstr (file, ".sig"))
+    yes = 1;
+
+  return yes;
+}
+  
+
 static int
 hash_data_file (char * file, int digest_algo, GCRY_MD_HD * r_md)
 {
@@ -219,8 +283,7 @@ hash_data_file (char * file, int digest_algo, GCRY_MD_HD * r_md)
   CDK_STREAM s;
   int rc;
 
-  if (strstr (file, ".asc") || strstr (file, ".gpg")
-      || strstr (file, ".pgp") || strstr (file, ".sig"))
+  if (is_openpgp_ext (file))
     file[strlen (file) - 4] = '\0';
   else
     return CDK_General_Error;
@@ -228,6 +291,7 @@ hash_data_file (char * file, int digest_algo, GCRY_MD_HD * r_md)
   rc = cdk_stream_open (file, &s);
   if (rc)
     return rc;
+  
   cdk_stream_set_hash_flag (s, digest_algo);
   cdk_stream_read (s, NULL, 0);
   mfx = _cdk_stream_get_opaque (s, fHASH);
@@ -236,6 +300,7 @@ hash_data_file (char * file, int digest_algo, GCRY_MD_HD * r_md)
       *r_md = gcry_md_copy (mfx->md);
       gcry_md_close (mfx->md);
     }
+  
   cdk_stream_close (s);
   return 0;
 }
@@ -258,7 +323,7 @@ handle_symkey_enc (CTX c, CDK_PACKET * pkt)
       return CDK_Out_Of_Core;
 
   key = pkt->pkt.symkey_enc;
-  c->dek = cdk_passphrase_to_dek (key->cipher_algo, &key->s2k, 0, pw);
+  c->dek = cdk_passphrase_to_dek (key->cipher_algo, key->s2k, 0, pw);
   if (!c->dek)
     rc = CDK_Error_No_Key;
   _cdk_passphrase_free (pw);
@@ -267,18 +332,22 @@ handle_symkey_enc (CTX c, CDK_PACKET * pkt)
 
 
 static int
-handle_pubkey_enc (CTX c, CDK_PACKET * pkt)
+handle_pubkey_enc (CTX c, CDK_HD hd, CDK_PACKET * pkt)
 {
   cdkPKT_pubkey_enc * enc;
   CDK_KBNODE snode, sk;
   int rc = 0;
 
   assert (pkt->pkttype == CDK_PKT_PUBKEY_ENC);
+
+  enc = pkt->pkt.pubkey_enc;
+  write_status (hd, CDK_CB_PUBKEY_ENC, "%08lX%08lX %d %d",
+                enc->keyid[0], enc->keyid[1], enc->pubkey_algo,
+                (enc->mpi[0]->bits+7)/8*8);
   
   if (c->dek)
     return 0; /* we already decrypted the session key */
 
-  enc = pkt->pkt.pubkey_enc;
   snode = _cdk_keydb_get_skblock (enc->keyid);
   if (!snode)
     return 0;
@@ -297,11 +366,14 @@ handle_pubkey_enc (CTX c, CDK_PACKET * pkt)
 static int
 handle_encrypted (CTX c, CDK_PACKET *pkt, int use_mdc)
 {
-  cdkPKT_encrypted *enc;
+  cdkPKT_encrypted * enc;
   int rc = 0;
 
   assert (pkt->pkttype == CDK_PKT_ENCRYPTED
           || pkt->pkttype == CDK_PKT_ENCRYPTED_MDC);
+
+  if (!c->dek)
+    return CDK_Error_No_Key;
   
   enc = pkt->pkt.encrypted;
   set_reset (c, enc->buf);
@@ -439,9 +511,9 @@ free_mainproc (CTX c)
 static int
 do_proc_packets (CDK_HD hd, CTX c, CDK_STREAM inp, CDK_STREAM out)
 {
-  CDK_PACKET *pkt = NULL;
+  CDK_PACKET * pkt = NULL;
   CDK_KBNODE n = NULL, node;
-  const char *s;
+  const char * s;
   int rc = 0, npos;
 
   if (!hd || !c)
@@ -471,8 +543,8 @@ do_proc_packets (CDK_HD hd, CTX c, CDK_STREAM inp, CDK_STREAM out)
       if (!c->node)
         c->node = n;
       else
-        cdk_kbnode_add (c->node, n);
-      
+        _cdk_kbnode_add (c->node, n);
+
       switch (pkt->pkttype)
         {
         case CDK_PKT_SYMKEY_ENC:
@@ -481,13 +553,13 @@ do_proc_packets (CDK_HD hd, CTX c, CDK_STREAM inp, CDK_STREAM out)
           break;
           
         case CDK_PKT_PUBKEY_ENC:
-          rc = handle_pubkey_enc (c, pkt);
+          rc = handle_pubkey_enc (c, hd, pkt);
           _cdk_log_debug (" handle_pubkey_enc (%d)\n", rc); 
           break;
           
         case CDK_PKT_ENCRYPTED_MDC: 
         case CDK_PKT_ENCRYPTED:
-          rc = handle_encrypted (c, pkt, pkt->pkttype==CDK_PKT_ENCRYPTED_MDC);
+          rc = handle_encrypted (c, pkt, pkt->pkttype ==CDK_PKT_ENCRYPTED_MDC);
           _cdk_log_debug (" handle_encrypted (%d)\n", rc);
           if (!rc)
             inp = c->s;
@@ -522,6 +594,9 @@ do_proc_packets (CDK_HD hd, CTX c, CDK_STREAM inp, CDK_STREAM out)
             c->sig.present = 1;
           break; /* handle it later */
 
+        case CDK_PKT_MDC:
+          break;
+
         default:
           rc = CDK_Inv_Packet;
           break; 
@@ -616,16 +691,11 @@ int
 cdk_file_decrypt (CDK_HD hd, const char *file, const char *output)
 {
   CDK_STREAM inp, out;
-  CDK_PACKET *pkt;
   int rc;
   
   if (!file || !output)
     return CDK_Inv_Value;
 
-  pkt = cdk_calloc (1, sizeof *pkt);
-  if (!pkt)
-    return CDK_Out_Of_Core;
-
   rc = cdk_stream_open (file, &inp);
   if (rc)
     return rc;
