@@ -450,7 +450,7 @@ cdk_keydb_search (CDK_KEYDB_HD kdb, CDK_DBSEARCH ks, CDK_KBNODE * r_key)
             rc = 0;
           if (rc)
             break;
-        }    
+        }
       if (!knode && rc == CDK_EOF)
         rc = CDK_Error_No_Key;
       if (rc)
@@ -534,6 +534,23 @@ cdk_keydb_get_bypattern (CDK_KEYDB_HD kdb, const char *patt,
 }
 
 
+static int
+keydb_check_key (CDK_PACKET * pkt)
+{
+  cdkPKT_public_key * pk;
+  
+  if (pkt->pkttype == CDK_PKT_PUBLIC_KEY
+      || pkt->pkttype == CDK_PKT_PUBLIC_SUBKEY)
+    pk = pkt->pkt.public_key;
+  else if (pkt->pkttype == CDK_PKT_SECRET_KEY
+      || pkt->pkttype == CDK_PKT_SECRET_SUBKEY)
+    pk = pkt->pkt.secret_key->pk;
+  else
+    return 0;
+  return !pk->is_revoked && !pk->has_expired;
+}
+
+
 static CDK_KBNODE
 keydb_find_byusage (CDK_KBNODE root, int req_usage, int pk)
 {
@@ -548,10 +565,12 @@ keydb_find_byusage (CDK_KBNODE root, int req_usage, int pk)
       pkttype = node->pkt->pkttype;
       if (pk && (node->pkt->pkttype == CDK_PKT_PUBLIC_KEY
                  || node->pkt->pkttype == CDK_PKT_PUBLIC_SUBKEY)
+          && keydb_check_key (node->pkt)
           && (node->pkt->pkt.public_key->pubkey_usage & req_usage))
 	return node;
       if (!pk && (node->pkt->pkttype == CDK_PKT_SECRET_KEY
                   || node->pkt->pkttype == CDK_PKT_SECRET_SUBKEY)
+          && keydb_check_key (node->pkt)
           && (node->pkt->pkt.secret_key->req_usage & req_usage))
 	return node;
     }
@@ -577,22 +596,22 @@ _cdk_keydb_find_bykeyid (CDK_KBNODE root, u32 * keyid)
 
 
 int
-cdk_keydb_get_sk_byname (const char *name, cdkPKT_secret_key ** ret_sk,
+cdk_keydb_get_sk_byname (const char * name, cdkPKT_secret_key ** ret_sk,
 			 int alloced)
 {
-  CDK_KBNODE knode, node = NULL;
-  cdkPKT_secret_key *sk = NULL;
-  CDK_KEYDB_HD khd;
+  CDK_KBNODE knode = NULL, node = NULL;
+  cdkPKT_secret_key * sk = NULL;
+  CDK_KEYDB_HD hd;
   int req_usage = 0;
   int rc = 0;
 
   if (!ret_sk)
     return CDK_Inv_Value;
 
-  khd = cdk_keydb_get_ctx (0, -1);
-  if (!khd)
+  hd = cdk_keydb_get_ctx (1, -1);
+  if (!hd)
     return CDK_Error_No_Keyring;
-  rc = cdk_keydb_get_bypattern (khd, name, &knode);
+  rc = cdk_keydb_get_bypattern (hd, name, &knode);
   if (rc)
     goto leave;
   if (alloced)
@@ -610,10 +629,12 @@ cdk_keydb_get_sk_byname (const char *name, cdkPKT_secret_key ** ret_sk,
 	  goto leave;
 	}
     }
-  _cdk_copy_seckey (&sk, node->pkt->pkt.secret_key);
+  sk = node->pkt->pkt.secret_key;
+  _cdk_kbnode_clone (node);
   cdk_kbnode_release (knode);
 
 leave:
+  cdk_keydb_search_reset (hd);
   *ret_sk = sk;
   return rc;
 }
@@ -625,7 +646,7 @@ cdk_keydb_get_pk_byname (const char *name, cdkPKT_public_key ** ret_pk,
 {
   CDK_KBNODE key, node = NULL;
   cdkPKT_public_key *pk = NULL;
-  CDK_KEYDB_HD khd;
+  CDK_KEYDB_HD hd;
   const char *s;
   int req_usage = 0;
   int rc = 0;
@@ -633,10 +654,10 @@ cdk_keydb_get_pk_byname (const char *name, cdkPKT_public_key ** ret_pk,
   if (!name || !ret_pk)
     return CDK_Inv_Value;
 
-  khd = cdk_keydb_get_ctx (0, -1);
-  if (!khd)
+  hd = cdk_keydb_get_ctx (0, -1);
+  if (!hd)
     return CDK_Error_No_Keyring;
-  rc = cdk_keydb_get_bypattern (khd, name, &key);
+  rc = cdk_keydb_get_bypattern (hd, name, &key);
   if (rc)
     goto leave;
   if (alloced)
@@ -670,14 +691,14 @@ cdk_keydb_get_pk_byname (const char *name, cdkPKT_public_key ** ret_pk,
   cdk_kbnode_release (key);
 
 leave:
+  cdk_keydb_search_reset (hd);
   *ret_pk = pk;
-
   return rc;
 }
 
 
 int
-cdk_keydb_get_pk (CDK_KEYDB_HD khd, u32 * keyid, cdkPKT_public_key ** r_pk)
+cdk_keydb_get_pk (CDK_KEYDB_HD hd, u32 * keyid, cdkPKT_public_key ** r_pk)
 {
   struct cdk_keydb_search_s ks;
   CDK_KBNODE key = NULL, node = NULL;
@@ -688,14 +709,14 @@ cdk_keydb_get_pk (CDK_KEYDB_HD khd, u32 * keyid, cdkPKT_public_key ** r_pk)
   if (!keyid || !r_pk)
     return CDK_Inv_Value;
 
-  if (khd == NULL)
+  if (hd == NULL)
     {
       keydb_hd = cdk_keydb_get_ctx (0, -1);
       if (!keydb_hd)
 	return CDK_Error_No_Keyring;
     }
   else
-    keydb_hd = khd;
+    keydb_hd = hd;
 
   if (!keyid[0])
     {
@@ -722,27 +743,26 @@ cdk_keydb_get_pk (CDK_KEYDB_HD khd, u32 * keyid, cdkPKT_public_key ** r_pk)
 
 leave:
   *r_pk = pk;
-
   return rc;
 }
 
 
 int
-cdk_keydb_get_sk (CDK_KEYDB_HD khd, u32 * keyid, cdkPKT_secret_key ** ret_sk)
+cdk_keydb_get_sk (CDK_KEYDB_HD hd, u32 * keyid, cdkPKT_secret_key ** ret_sk)
 {
-  CDK_KEYDB_HD hd;
+  CDK_KEYDB_HD khd;
   CDK_KBNODE snode, node;
   cdkPKT_secret_key *sk = NULL;
   int rc = 0;
 
-  if (!khd)
+  if (!hd)
     {
-      hd = cdk_keydb_get_ctx (1, -1);
-      if (!hd)
+      khd = cdk_keydb_get_ctx (1, -1);
+      if (!khd)
 	return CDK_Error_No_Keyring;
     }
   else
-    hd = khd;
+    khd = hd;
   rc = cdk_keydb_get_bykeyid (hd, keyid, &snode);
   if (rc)
     goto leave;
@@ -754,7 +774,8 @@ cdk_keydb_get_sk (CDK_KEYDB_HD khd, u32 * keyid, cdkPKT_secret_key ** ret_sk)
       goto leave;
     }
 
-  _cdk_copy_seckey (&sk, node->pkt->pkt.secret_key);
+  sk = node->pkt->pkt.secret_key;
+  _cdk_kbnode_clone (node);
   cdk_kbnode_release (snode);
 
 leave:
@@ -857,18 +878,18 @@ keydb_merge_selfsig (CDK_KBNODE key)
 
 
 static void
-key_set_expire (cdkPKT_public_key * pk, u32 pk_expired)
+keydb_set_expire (cdkPKT_public_key * pk, u32 pk_expired)
 {
-  u32 key_expire = 0;
   u32 curtime = 0;
 
   if (pk)
     {
-      key_expire = 0;
-      curtime = make_timestamp ();
-      pk->expiredate = pk->timestamp + pk_expired;
-      key_expire = pk->expiredate;
-      pk->has_expired = (key_expire >= curtime) ? 0 : 1;
+      curtime = _cdk_timestamp ();
+      if (pk->version == 3)
+          pk->expiredate = (pk->expiredate * 86400) + pk->timestamp;          
+      else
+          pk->expiredate = pk->timestamp + pk_expired;
+      pk->has_expired = pk->expiredate >= curtime? 1 : 0;
     }
 }
 
@@ -885,7 +906,8 @@ keydb_parse_allsigs (CDK_KBNODE key, int check)
   if (!key)
     return CDK_Inv_Value;
 
-  if (cdk_kbnode_find (key, CDK_PKT_SECRET_KEY))
+  kb = cdk_kbnode_find (key, CDK_PKT_SECRET_KEY);
+  if (kb)
     return 0;
 
   /* reset */
@@ -954,7 +976,7 @@ keydb_parse_allsigs (CDK_KBNODE key, int check)
 		  if (s)
 		    {
 		      pk_expired = buffer_to_u32 (s->d);
-		      key_set_expire (kb->pkt->pkt.public_key, pk_expired);
+		      keydb_set_expire (kb->pkt->pkt.public_key, pk_expired);
 		    }
 		  if (check)
 		    ;
@@ -974,7 +996,7 @@ keydb_parse_allsigs (CDK_KBNODE key, int check)
 		  if (s)
 		    {
 		      pk_expired = buffer_to_u32 (s->d);
-		      key_set_expire (kb->pkt->pkt.public_key, pk_expired);
+		      keydb_set_expire (kb->pkt->pkt.public_key, pk_expired);
 		    }
 		  if (check)
 		    ;
@@ -1059,7 +1081,7 @@ cdk_keydb_get_keyblock (CDK_STREAM inp, CDK_KBNODE * r_knode)
       if (!knode)
 	knode = node;
       else
-	cdk_kbnode_add (knode, node);
+	_cdk_kbnode_add (knode, node);
     }
 
   if (got_key)
@@ -1114,19 +1136,35 @@ cdk_keydb_export (CDK_STREAM out, CDK_STRLIST remusr, int secret)
 }
 
 
-int
-cdk_keydb_import (CDK_KEYDB_HD hd, CDK_KBNODE knode)
+static CDK_PACKET *
+find_key_packet (CDK_KBNODE knode, int * r_secret)
 {
-  CDK_KBNODE node, chk = NULL;
   CDK_PACKET * pkt;
-  CDK_STREAM inp;
-  u32 keyid[2];
-  size_t len;
-  int rc = 0;
 
   pkt = cdk_kbnode_find_packet (knode, CDK_PKT_PUBLIC_KEY);
+  if (!pkt)
+    {
+      pkt = cdk_kbnode_find_packet (knode, CDK_PKT_SECRET_KEY);
+      *r_secret = pkt? 1 : 0;
+    }
+  return pkt;
+}
+
+
+int
+cdk_keydb_import (CDK_KEYDB_HD hd, CDK_KBNODE knode, int result[4])
+{
+  CDK_KBNODE node, chk = NULL;
+  CDK_PACKET * pkt;
+  CDK_STREAM out;
+  u32 keyid[2];
+  int rc = 0, is_sk = 0;
+
+  memset (result, 0, 4 * sizeof (int));
+  pkt = find_key_packet (knode, &is_sk);
   if (!pkt)
       return CDK_Inv_Packet;
+  result[is_sk] = 1;
   cdk_pk_get_keyid (pkt->pkt.public_key, keyid);
   cdk_keydb_get_bykeyid (hd, keyid, &chk);
   if (chk)
@@ -1136,30 +1174,23 @@ cdk_keydb_import (CDK_KEYDB_HD hd, CDK_KBNODE knode)
     }
   
   cdk_keydb_search_reset (hd);
-  inp = hd->buf;
-  if (!inp)
-    {
-      rc = cdk_stream_open (hd->name, &inp);
-      if (rc)
-        return rc;
-    }
-  
-  len = cdk_stream_get_length (inp);
-  if (len > 0)
-    cdk_stream_seek (inp, len);
+  if (hd->buf)
+    return CDK_Inv_Mode;
+  rc = _cdk_stream_append (hd->name, &out);
+  if (rc)
+    return rc;
   
   for (node = knode; node; node = node->next)
     {
       if (node->pkt->pkttype == CDK_PKT_RING_TRUST)
         continue; /* No uniformed syntax for this packet */
-      rc = cdk_pkt_build (inp, node->pkt);
+      rc = cdk_pkt_build (out, node->pkt);
       if (rc)
         break;
     }
-  
-  if (!hd->buf)
-    cdk_stream_close (inp);
-
+  if (!rc)
+    result[is_sk? 3 : 2] = 1;
+  cdk_stream_close (out);
   return rc;
 }  
 
