@@ -47,7 +47,7 @@ cdk_pk_get_fingerprint (cdkPKT_public_key * pk, byte * fpr)
   hd = gcry_md_open (md_algo, 0);
   if (!hd)
     return CDK_Gcry_Error;
-  _cdk_hash_pubkey (pk, hd);
+  _cdk_hash_pubkey (pk, hd, 1);
   gcry_md_final (hd);
   memcpy (fpr, gcry_md_read (hd, md_algo), dlen);
   gcry_md_close (hd);
