@@ -61,7 +61,7 @@ pklist_select_mdc (CDK_KEYLIST pkl)
 
   for (pkr = pkl; pkr; pkr = pkr->next)
     {
-      if (pkr->key.pk->uid)	/* selected by user ID */
+      if (pkr->key.pk->uid) /* selected by user ID */
 	mdc = pkr->key.pk->uid->mdc_feature;
       if (!mdc)
 	return 0; /* at least on recipeint does not support it */
@@ -204,10 +204,7 @@ cdk_pklist_select_algo (CDK_KEYLIST pkl)
 int
 cdk_pklist_use_mdc (CDK_KEYLIST pkl)
 {
-  if (!pkl)
-    return 0;
-
-  return pklist_select_mdc (pkl);
+  return pkl? pklist_select_mdc (pkl) : 0;
 }
 
 
@@ -272,7 +269,6 @@ leave:
       pkl = NULL;
     }
   *ret_pkl = pkl;
-
   return rc;
 }
 
@@ -297,8 +293,7 @@ cdk_pklist_encrypt (CDK_KEYLIST pkl, CDK_DEK dek, CDK_STREAM outp)
     {
       pk = pkl->key.pk;
       cdk_free (enc);
-      enc = NULL;
-      enc = calloc (1, sizeof *enc);
+      enc = cdk_calloc (1, sizeof *enc);
       if (!enc)
 	return CDK_Out_Of_Core;
       enc->pubkey_algo = pk->pubkey_algo;
@@ -355,7 +350,7 @@ cdk_sklist_build (CDK_STRLIST locusr, CDK_KEYLIST * ret_skl, int unlock,
 		  unsigned int use)
 {
   CDK_KEYLIST r = NULL, skl = NULL;
-  cdkPKT_secret_key *sk = NULL;
+  cdkPKT_secret_key * sk = NULL;
   int rc = 0;
 
   if (!ret_skl)
@@ -375,6 +370,12 @@ cdk_sklist_build (CDK_STRLIST locusr, CDK_KEYLIST * ret_skl, int unlock,
 	}
       else
 	{
+          if (unlock)
+            {
+              rc = _cdk_seckey_unprotect2 (sk);
+              if (rc)
+                goto fail;
+            }
 	  r = cdk_calloc (1, sizeof *r);
 	  if (!r)
 	    return CDK_Out_Of_Core;
@@ -404,6 +405,12 @@ cdk_sklist_build (CDK_STRLIST locusr, CDK_KEYLIST * ret_skl, int unlock,
 	    }
 	  else
 	    {
+              if (unlock)
+                {
+                  rc = _cdk_seckey_unprotect2 (sk);
+                  if (rc)
+                    goto fail; 
+                }
 	      r = cdk_calloc (1, sizeof *r);
 	      if (!r)
 		return CDK_Out_Of_Core;
@@ -415,6 +422,7 @@ cdk_sklist_build (CDK_STRLIST locusr, CDK_KEYLIST * ret_skl, int unlock,
 	}
     }
 
+ fail:
   if (rc)
     {
       cdk_sklist_release (skl);
@@ -475,7 +483,7 @@ cdk_sklist_write_onepass (CDK_KEYLIST skl, CDK_STREAM outp, int sigclass)
 
 int
 cdk_sklist_write (CDK_KEYLIST skl, CDK_STREAM outp, GCRY_MD_HD hash,
-		  int sigclass)
+		  int sigclass, int sigver)
 {
   CDK_KEYLIST r = NULL;
   cdkPKT_signature *sig = NULL;
@@ -492,6 +500,7 @@ cdk_sklist_write (CDK_KEYLIST skl, CDK_STREAM outp, GCRY_MD_HD hash,
       sig = cdk_calloc (1, sizeof *sig);
       if (!sig)
 	return CDK_Out_Of_Core;
+      sig->version = sigver;
       _cdk_sig_create (r->key.sk->pk, sig);
       sig->sig_class = sigclass;
       md = gcry_md_copy (hash);
