@@ -37,6 +37,18 @@
 
 #define init_packet(ctx) memset ((ctx), 0, sizeof (ctx))
 
+#define wipemem(_ptr,_len) \
+do \
+{ \
+  volatile char *_vptr = (volatile char *)(_ptr); \
+  size_t _vlen = (_len); \
+  while (_vlen) \
+    { \
+      *_vptr = 0; \
+      _vptr++; \
+      _vlen--; \
+    } \
+} while (0)
 
 /*-- main.c --*/
 int _cdk_get_log_level (void);
@@ -46,7 +58,7 @@ char * _cdk_passphrase_get (const char * prompt);
 void _cdk_passphrase_free (char * pw);
 
 /*-- misc.c --*/
-u32 make_timestamp (void);
+u32 _cdk_timestamp (void);
 u32 buffer_to_u32 (const byte * buf);
 void u32_to_buffer (u32 u, byte * buf);
 const char* ascii_memistr (const char * buf, size_t buflen, const char * sub);
@@ -54,7 +66,7 @@ const char* ascii_memistr (const char * buf, size_t buflen, const char * sub);
 /*-- pubkey.c --*/
 int _cdk_pk_algo_usage (int algo);
 int _cdk_pk_algo_usage (int algo);
-int _cdk_pk_test_algo (int algo, unsigned int usage_flags);
+int _cdk_pk_test_algo (int algo, unsigned int usage);
 
 /*-- cipher.c --*/
 int _cdk_cipher_test_algo (int algo);
@@ -63,34 +75,27 @@ int _cdk_md_test_algo (int algo);
 /*-- new-packet.c --*/
 byte * _cdk_subpkt_get_array (CDK_SUBPKT s, int count, size_t * r_nbytes);
 int _cdk_subpkt_add (CDK_SUBPKT root, CDK_SUBPKT node);
-int _cdk_subpkt_copy (struct cdk_subpkt_s ** r_dst, struct cdk_subpkt_s * src);
+int _cdk_subpkt_copy (CDK_SUBPKT * r_dst, CDK_SUBPKT src);
 int _cdk_subpkt_hash (CDK_SUBPKT hashed, size_t * r_nbytes, GCRY_MD_HD hd);
 
 /*-- sig-check.c --*/
-void _cdk_hash_pubkey (cdkPKT_public_key * pk, GCRY_MD_HD hd);
+void _cdk_hash_pubkey (cdkPKT_public_key * pk, GCRY_MD_HD md, int use_fpr);
 int _cdk_signature_check (cdkPKT_public_key * pk, cdkPKT_signature *sig,
                           GCRY_MD_HD digest, int * r_expired);
 void _cdk_hash_sig_data (cdkPKT_signature * sig, GCRY_MD_HD hd);
 
 /*-- kbnode.c --*/
-void cdk_kbnode_delete (CDK_KBNODE node);
-void cdk_kbnode_add (CDK_KBNODE root, CDK_KBNODE node);
-void cdk_kbnode_insert (CDK_KBNODE root, CDK_KBNODE node, int pkttype);
-CDK_KBNODE cdk_kbnode_find_prev (CDK_KBNODE root, CDK_KBNODE node, int pktype);
-CDK_KBNODE cdk_kbnode_find_next (CDK_KBNODE node, int pkttype);
-CDK_KBNODE cdk_kbnode_find (CDK_KBNODE node, int pkttype);
-CDK_KBNODE cdk_kbnode_walk (CDK_KBNODE root, CDK_KBNODE * context, int all);
-
-#define is_deleted_kbnode(a)  ((a)->private_flag & 1)
-#define is_cloned_kbnode(a)   ((a)->private_flag & 2)
+void _cdk_kbnode_add (CDK_KBNODE root, CDK_KBNODE node);
+void _cdk_kbnode_clone (CDK_KBNODE node);
 
 /*-- sesskey.c --*/
 int _cdk_pkcs1_sesskey (GCRY_MPI * esk, CDK_DEK dek, int nbits);
 int _cdk_pkcs1_digest (byte ** r_md, size_t * r_mdlen, int pk_algo,
                        const byte * md, int digest_algo, unsigned nbits);
+int _cdk_seckey_unprotect2 (cdkPKT_secret_key * sk);
 
 /*-- seckey.c --*/
-int cdk_hash_passphrase (CDK_DEK dek, char * pw, CDK_S2K_HD * s2k,int creat);
+int cdk_hash_passphrase (CDK_DEK dek, char * pw, CDK_S2K_HD s2k, int creat);
 
 /*-- keydb.c --*/
 u32 _cdk_keyid_get (CDK_PACKET * pkt, u32 * keyid);
@@ -101,14 +106,16 @@ CDK_KBNODE _cdk_keydb_get_skblock (u32 * keyid);
 CDK_KBNODE _cdk_keydb_find_bykeyid (CDK_KBNODE root, u32 * keyid);
 
 /*-- sign.c --*/
-int _cdk_sig_create (cdkPKT_public_key * pk, cdkPKT_signature *sig);
+int _cdk_sig_create (cdkPKT_public_key * pk, cdkPKT_signature * sig);
 int _cdk_sig_hash_for (int pubkey_algo, int pkt_version);
 void _cdk_trim_string (char * s, int canon);
 
 /*-- stream.c --*/
 void * _cdk_stream_get_opaque (CDK_STREAM s, int id);
 const char * _cdk_stream_get_fname (CDK_STREAM s);
+FILE * _cdk_stream_get_fp (CDK_STREAM s);
 int _cdk_stream_gets (CDK_STREAM s, char * buf, size_t count);
+int _cdk_stream_append (const char * file, CDK_STREAM * ret_s);
 
 /*-- verify.c --*/
 void _cdk_result_verify_free (Verify_Result res);
