@@ -57,7 +57,18 @@ cdk_pkt_new (CDK_PACKET ** r_pkt)
 
 
 static void
-free_pubkey_enc (cdkPKT_pubkey_enc *enc)
+free_symkey_enc (cdkPKT_symkey_enc * enc)
+{
+  if (enc)
+    {
+      cdk_free (enc->s2k);
+      cdk_free (enc);
+    }
+}
+
+
+static void
+free_pubkey_enc (cdkPKT_pubkey_enc * enc)
 {
   int nenc;
   if (enc)
@@ -147,6 +158,7 @@ _cdk_free_seckey (cdkPKT_secret_key * sk)
       nskey = cdk_pk_get_nskey (sk->pubkey_algo);
       while (sk->mpi && nskey--)
         {
+          wipemem (sk->mpi[nskey], sk->mpi[nskey]->bytes);
           cdk_free (sk->mpi[nskey]);
           sk->mpi[nskey] = NULL; 
         }
@@ -154,6 +166,7 @@ _cdk_free_seckey (cdkPKT_secret_key * sk)
       sk->encdata = NULL;
       _cdk_free_pubkey (sk->pk);
       sk->pk = NULL;
+      cdk_free (sk->protect.s2k);
       cdk_free (sk);
     }
 }
@@ -187,7 +200,7 @@ cdk_pkt_free (CDK_PACKET * pkt)
     case CDK_PKT_SECRET_SUBKEY: _cdk_free_seckey (pkt->pkt.secret_key); break;
     case CDK_PKT_SIGNATURE    : _cdk_free_signature (pkt->pkt.signature); break;
     case CDK_PKT_PUBKEY_ENC   : free_pubkey_enc (pkt->pkt.pubkey_enc); break;
-    case CDK_PKT_SYMKEY_ENC   : cdk_free (pkt->pkt.symkey_enc); break;
+    case CDK_PKT_SYMKEY_ENC   : free_symkey_enc (pkt->pkt.symkey_enc); break;
     case CDK_PKT_MDC          : cdk_free (pkt->pkt.mdc); break;
     case CDK_PKT_ENCRYPTED    :
     case CDK_PKT_ENCRYPTED_MDC: free_encrypted (pkt->pkt.encrypted); break;
@@ -364,7 +377,9 @@ _cdk_copy_pubkey (cdkPKT_public_key ** dst, cdkPKT_public_key * src)
 int
 _cdk_copy_seckey (cdkPKT_secret_key ** dst, cdkPKT_secret_key * src)
 {
-  cdkPKT_secret_key *k;
+  cdkPKT_secret_key * k;
+  CDK_MPI a;
+  CDK_S2K_HD s2k;
   int i;
 
   if (!dst || !src)
@@ -376,15 +391,31 @@ _cdk_copy_seckey (cdkPKT_secret_key ** dst, cdkPKT_secret_key * src)
   memcpy (k, src, sizeof *k);
   _cdk_copy_pubkey (&k->pk, src->pk);
 
+  if (src->encdata)
+    {
+      k->encdata = cdk_calloc (1, src->enclen + 1);
+      if (!k->encdata)
+        return CDK_Out_Of_Core;
+      memcpy (k->encdata, src->encdata, src->enclen);
+    }
+
+  s2k = k->protect.s2k = cdk_calloc (1, sizeof *k->protect.s2k);
+  if (!k->protect.s2k)
+    return CDK_Out_Of_Core;
+  s2k->mode = src->protect.s2k->mode;
+  s2k->hash_algo = src->protect.s2k->hash_algo;
+  s2k->count = src->protect.s2k->count;
+  memcpy (s2k->salt, src->protect.s2k->salt, 8);
+
   for (i = 0; i < cdk_pk_get_nskey (src->pubkey_algo); i++)
     {
-      k->mpi[i] = cdk_calloc (1, sizeof **k->mpi + src->mpi[i]->bytes + 2);
+      a = k->mpi[i] = cdk_calloc (1, sizeof **k->mpi + src->mpi[i]->bytes + 2);
       if (!k->mpi[i])
 	return CDK_Out_Of_Core;
-      k->mpi[i]->bits = src->mpi[i]->bits;
-      k->mpi[i]->bytes = src->mpi[i]->bytes;
+      a->bits = src->mpi[i]->bits;
+      a->bytes = src->mpi[i]->bytes;
       /* copy 2 extra bytes (prefix) */
-      memcpy (k->mpi[i]->data, src->mpi[i]->data, src->mpi[i]->bytes + 2);
+      memcpy (a->data, src->mpi[i]->data, src->mpi[i]->bytes + 2);
     }
   *dst = k;
 
