@@ -21,9 +21,9 @@
 #ifndef OPENCDK_H
 #define OPENCDK_H
 
-#include <gcrypt.h>
+#define OPENCDK_VERSION "0.3.3"
 
-#define OPENCDK_VERSION "0.3.2"
+#include <gcrypt.h>
 
 /* General contexts */
 struct cdk_handle_s;
@@ -38,6 +38,9 @@ typedef struct cdk_mpi_s *CDK_MPI;
 struct cdk_dek_s;
 typedef struct cdk_dek_s *CDK_DEK;
 
+struct cdk_s2k_hd_s;
+typedef struct cdk_s2k_hd_s *CDK_S2K_HD;
+
 struct cdk_stream_context_s;
 typedef struct cdk_stream_context_s *CDK_STREAM;
 
@@ -86,7 +89,8 @@ typedef enum
   CDK_Inv_Mode = 20,
   CDK_Error_No_Keyring = 21,
   CDK_Wrong_Format = 22,
-  CDK_Inv_Packet_Ver = 23
+  CDK_Inv_Packet_Ver = 23,
+  CDK_Too_Short = 24
 } cdk_err_t;
 
 enum
@@ -103,6 +107,16 @@ enum
   CDK_COMPRESS_ZLIB = 2
 };
 
+enum
+{
+  CDK_PK_RSA = 1,
+  CDK_PK_RSA_E = 2,
+  CDK_PK_RSA_S = 3,
+  CDK_PK_ELG_E = 16,
+  CDK_PK_DSA   = 17,
+  CDK_PK_ELG   = 20
+};
+
 enum
 {
   CDK_ARMOR_MESSAGE   = 0,
@@ -169,7 +183,14 @@ enum
   CDK_ATTR_ALGO_PK  = 6,
   CDK_ATTR_ALGO_MD  = 7
 };
-  
+
+
+enum
+{
+  CDK_CB_NONE       = 0,
+  CDK_CB_PUBKEY_ENC = 1
+};
+
 
 typedef enum
 {
@@ -195,15 +216,6 @@ typedef enum
 } cdk_packet_types_t;
 
 
-typedef struct
-{
-  int mode;
-  unsigned char hash_algo;
-  unsigned char salt[8];
-  unsigned long count;  
-} CDK_S2K_HD;
-
-
 typedef struct
 {
   unsigned long len;
@@ -411,7 +423,12 @@ void cdk_handle_set_s2k (CDK_HD hd, int mode, int digest, int cipher);
 void cdk_handle_set_cipher (CDK_HD hd, int cipher);
 void cdk_handle_set_digest (CDK_HD hd, int digest);
 void cdk_handle_set_compress (CDK_HD hd, int algo);
-     
+void cdk_handle_set_armor (CDK_HD hd, int val);
+void cdk_handle_set_compat (CDK_HD hd, int val);
+void cdk_handle_set_callback (CDK_HD hd,
+                              void (*cb) (void *opa, int type, const char * s),
+                              void * cb_value);
+
 /*-- new-packet.c --*/
 int cdk_pkt_new (CDK_PACKET ** r_pkt);
 int cdk_pkt_alloc (CDK_PACKET ** r_pkt, int id);
@@ -427,16 +444,16 @@ const unsigned char * cdk_subpkt_get_buf (CDK_SUBPKT ctx,
                                           size_t * r_nbytes);
 
 /*-- pubkey.c --*/
-#define is_RSA(a) ((a) == GCRY_PK_RSA \
-                   || (a) == GCRY_PK_RSA_E \
-                   || (a) == GCRY_PK_RSA_S)
-#define is_ELG(a) ((a) == GCRY_PK_ELG || (a) == GCRY_PK_ELG_E)
-#define is_DSA(a) ((a) == GCRY_PK_DSA)
+#define is_RSA(a) ((a) == CDK_PK_RSA \
+                   || (a) == CDK_PK_RSA_E \
+                   || (a) == CDK_PK_RSA_S)
+#define is_ELG(a) ((a) == CDK_PK_ELG || (a) == CDK_PK_ELG_E)
+#define is_DSA(a) ((a) == CDK_PK_DSA)
 
 int cdk_pk_encrypt (cdkPKT_public_key * pk, cdkPKT_pubkey_enc * pke,
                     GCRY_MPI esk);
 int cdk_pk_decrypt (cdkPKT_secret_key * sk, cdkPKT_pubkey_enc * pke,
-                    GCRY_MPI *r_sk);
+                    GCRY_MPI * r_sk);
 int cdk_pk_sign (cdkPKT_secret_key * sk, cdkPKT_signature * sig,
                  const unsigned char * md);
 int cdk_pk_verify (cdkPKT_public_key * pk, cdkPKT_signature * sig,
@@ -458,17 +475,19 @@ const void * cdk_sig_get_data_attr (CDK_HD hd, int idx, int what);
 /*-- seckey.c --*/
 typedef char * (*CDK_PASSPHRASE_CB) (const char *);
 int cdk_seckey_unprotect (cdkPKT_secret_key * sk, char * pw);
-CDK_DEK cdk_passphrase_to_dek (int cipher_algo, CDK_S2K_HD * s2k,
+CDK_DEK cdk_passphrase_to_dek (int cipher_algo, CDK_S2K_HD s2k,
                                int mode, char * pw);
 void cdk_passphrase_cb_set (CDK_PASSPHRASE_CB pass_fnc);
 
 /*-- seskey.c --*/
 CDK_DEK cdk_pkcs1_to_dek (cdkPKT_pubkey_enc * enc, cdkPKT_secret_key * sk);
 int cdk_dek_new (CDK_DEK * r_dek, int algo);
+int cdk_s2k_new (CDK_S2K_HD * ret_s2k, int mode, int algo,
+                 const unsigned char * salt);
 
 /*-- armor.c --*/
 int cdk_file_armor (CDK_HD hd, const char * file, const char * output);
-int cdk_file_dearmor (const char * file, const char *output);
+int cdk_file_dearmor (const char * file, const char * output);
 int cdk_armor_filter_use (CDK_STREAM inp);
 
 /*-- stream.c --*/
@@ -476,8 +495,10 @@ int cdk_stream_control (CDK_STREAM s, int ctl, int val);
 int cdk_stream_open (const char * file, CDK_STREAM * ret_s);
 int cdk_stream_new (const char * file, CDK_STREAM * ret_s);
 CDK_STREAM cdk_stream_tmp (void);
-CDK_STREAM cdk_stream_tmp_from_mem (const unsigned char *buf, size_t count);
+CDK_STREAM cdk_stream_tmp_from_mem (const unsigned char * buf, size_t count);
+void cdk_stream_tmp_set_mode (CDK_STREAM s, int val);
 CDK_STREAM cdk_stream_fpopen (FILE * fp, unsigned write_mode);
+int cdk_stream_flush (CDK_STREAM s);
 int cdk_stream_set_cache (CDK_STREAM s, int val);
 int cdk_stream_filter_disable (CDK_STREAM s, int type);
 int cdk_stream_close (CDK_STREAM s);
@@ -494,7 +515,7 @@ int cdk_stream_set_literal_flag (CDK_STREAM s, int mode, const char * fname);
 int cdk_stream_set_cipher_flag (CDK_STREAM s, CDK_DEK dek, int use_mdc);
 int cdk_stream_set_compress_flag (CDK_STREAM s, int algo);
 int cdk_stream_set_hash_flag (CDK_STREAM s, int algo);
-int cdk_stream_set_text_flag (CDK_STREAM s, const char *lf);
+int cdk_stream_set_text_flag (CDK_STREAM s, const char * lf);
 int cdk_stream_kick_off (CDK_STREAM inp, CDK_STREAM out);
 
 /*-- keydb.c --*/
@@ -503,34 +524,40 @@ int cdk_keydb_new (CDK_KEYDB_HD * r_hd, int type,
 int cdk_keydb_open (CDK_KEYDB_HD kdb, CDK_STREAM * ret_kr);
 int cdk_keydb_add_resource (const char * name, int is_secret);
 int cdk_keydb_remove_resource (int id);
-const char* cdk_keydb_get_name (int id);
+const char * cdk_keydb_get_name (int id);
 int cdk_keydb_is_secret (int id);
 int cdk_keydb_find_idx (int is_secret, int * r_pos);
 CDK_KEYDB_HD cdk_keydb_get_ctx (int is_secret, int id);
 int cdk_keydb_search_new (CDK_DBSEARCH * r_dbs, int type, void * desc);
 void cdk_keydb_search_free (CDK_DBSEARCH dbs);
-int cdk_keydb_search (CDK_KEYDB_HD kdb, CDK_DBSEARCH ks, CDK_KBNODE *r_key);
+int cdk_keydb_search (CDK_KEYDB_HD kdb, CDK_DBSEARCH ks, CDK_KBNODE * r_key);
 void cdk_keydb_search_reset (CDK_KEYDB_HD hd);
-int cdk_keydb_get_bykeyid (CDK_KEYDB_HD kdb, unsigned long *keyid,
+int cdk_keydb_get_bykeyid (CDK_KEYDB_HD kdb, unsigned long * keyid,
                            CDK_KBNODE * r_key);
 int cdk_keydb_get_byfpr (CDK_KEYDB_HD kdb, const unsigned char * fpr,
                          CDK_KBNODE * r_key);
 int cdk_keydb_get_bypattern (CDK_KEYDB_HD kdb, const char * patt,
                              CDK_KBNODE * r_pkt);
-int cdk_keydb_get_pk (CDK_KEYDB_HD khd, unsigned long *keyid,
+int cdk_keydb_get_pk (CDK_KEYDB_HD khd, unsigned long * keyid,
                       cdkPKT_public_key ** ret_pk);
-int cdk_keydb_get_sk (CDK_KEYDB_HD khd, unsigned long *keyid,
+int cdk_keydb_get_sk (CDK_KEYDB_HD khd, unsigned long * keyid,
                       cdkPKT_secret_key ** ret_sk);
-int cdk_keydb_get_pk_byname (const char * name, cdkPKT_public_key **ret_pk,
+int cdk_keydb_get_pk_byname (const char * name, cdkPKT_public_key ** ret_pk,
                              int alloced);
-int cdk_keydb_get_sk_byname (const char * name, cdkPKT_secret_key **ret_sk,
+int cdk_keydb_get_sk_byname (const char * name, cdkPKT_secret_key ** ret_sk,
                              int alloced);
 int cdk_keydb_get_keyblock (CDK_STREAM inp, CDK_KBNODE * ret_key);
 int cdk_keydb_export (CDK_STREAM out, CDK_STRLIST remusr, int secret);
-int cdk_keydb_import (CDK_KEYDB_HD hd, CDK_KBNODE knode);
+int cdk_keydb_import (CDK_KEYDB_HD hd, CDK_KBNODE knode, int result[4]);
 
 /*-- kbnode.c --*/
 CDK_KBNODE cdk_kbnode_new (CDK_PACKET * pkt);
+int cdk_kbnode_read_from_mem (CDK_KBNODE * ret_node,
+                             const unsigned char * buf,
+                             size_t buflen);
+int cdk_kbnode_write_to_mem (CDK_KBNODE node,
+                             unsigned char * buf,
+                             size_t * r_nbytes);
 void cdk_kbnode_release (CDK_KBNODE n);
 CDK_KBNODE cdk_kbnode_walk (CDK_KBNODE root, CDK_KBNODE * context, int all);
 CDK_PACKET * cdk_kbnode_find_packet (CDK_KBNODE node, int pkttype);
@@ -544,8 +571,8 @@ int cdk_pk_get_fingerprint (cdkPKT_public_key * pk, unsigned char * fpr);
 unsigned long cdk_pk_fingerprint_get_keyid (const unsigned char * fpr,
                                             size_t fprlen,
                                             unsigned long * keyid);
-unsigned long cdk_pk_get_keyid (cdkPKT_public_key * pk, unsigned long *keyid);
-unsigned long cdk_sk_get_keyid (cdkPKT_secret_key * sk, unsigned long *keyid);
+unsigned long cdk_pk_get_keyid (cdkPKT_public_key * pk, unsigned long * keyid);
+unsigned long cdk_sk_get_keyid (cdkPKT_secret_key * sk, unsigned long * keyid);
 unsigned long cdk_sig_get_keyid (cdkPKT_signature * sig, unsigned long *keyid);
 
 /*-- sig-check.c --*/
@@ -561,10 +588,13 @@ int cdk_sklist_build (CDK_STRLIST locusr, CDK_KEYLIST * ret_skl, int unlock,
                       unsigned int use);
 void cdk_sklist_release (CDK_KEYLIST skl);
 int cdk_sklist_write (CDK_KEYLIST skl, CDK_STREAM outp, GCRY_MD_HD hash,
-                      int sigclass);
+                      int sigclass, int sigver);
 int cdk_sklist_write_onepass (CDK_KEYLIST skl, CDK_STREAM outp, int sigclass);
 
 /*-- encrypt.c --*/
+int cdk_stream_encrypt (CDK_HD hd, CDK_STRLIST remusr,
+                        CDK_STREAM inp,
+                        CDK_STREAM out);
 int cdk_file_encrypt (CDK_HD hd, CDK_STRLIST remusr,
                       const char * file,
                       const char * output);
@@ -577,6 +607,9 @@ int cdk_file_verify_clearsign (CDK_HD hd, const char * file,
                                const char * output);
 
 /*-- sign.c --*/
+int cdk_file_sign (CDK_HD hd, CDK_STRLIST locusr, CDK_STRLIST remusr,
+                   const char * file, const char * output,
+                   int detached, int encryptflag);
 int cdk_file_clearsign (CDK_HD hd, CDK_STRLIST locusr,
                         const char * file, const char * output);
 
@@ -591,6 +624,10 @@ CDK_STRLIST cdk_strlist_add (CDK_STRLIST * list, const char * string);
 const char * cdk_strlist_walk (CDK_STRLIST root, CDK_STRLIST * context);
 const char* cdk_check_version (const char * req_version);
 
+/* UTF8 */
+char * cdk_utf8_encode (const char * string);
+char * cdk_utf8_decode (const char * string, size_t length, int delim);
+
 /*-- keyserver.c --*/
 int cdk_keyserver_recv_key (const char * host, int port,
                             unsigned long * keyid, CDK_KBNODE * r_knode);
