@@ -84,7 +84,6 @@ plaintext_decode (void * opaque, FILE * in, FILE * out)
   cdk_stream_close (si);
   cdk_stream_close (so);
   cdk_pkt_free (&pkt);
-  
   return rc;
 }
 
@@ -121,7 +120,7 @@ plaintext_encode (void * opaque, FILE * in, FILE * out)
   memcpy (pt->name, pfx->filename, filelen);
   pt->namelen = filelen;
   pt->name[pt->namelen] = '\0';
-  pt->timestamp = make_timestamp ();
+  pt->timestamp = _cdk_timestamp ();
   pt->mode = pfx->mode ? 't' : 'b';
   pt->len = cdk_stream_get_length (si);
   pt->buf = si;
