@@ -91,7 +91,7 @@ skey_to_sexp (GCRY_SEXP * r_skey, cdkPKT_secret_key * sk)
   cdkPKT_public_key *pk = NULL;
   GCRY_SEXP sexp = NULL;
   GCRY_MPI * m_pk = NULL, * m_sk = NULL;
-  const char *fmt = NULL;
+  const char * fmt = NULL;
   int ncount = 0, nscount = 0;
   int rc = 0;
 
@@ -147,8 +147,8 @@ static int
 key_to_sexp (GCRY_SEXP * r_key, cdkPKT_public_key * pk)
 {
   GCRY_SEXP sexp = NULL;
-  GCRY_MPI *m = NULL;
-  const char *fmt = NULL;
+  GCRY_MPI * m = NULL;
+  const char * fmt = NULL;
   int ncount = 0;
   int rc = 0;
 
@@ -227,7 +227,7 @@ md_to_sexp (GCRY_SEXP * r_md, int algo, const byte * md, size_t mdlen)
 
 
 static int
-sexp_to_bitmpi (GCRY_SEXP sexp, const char *val, CDK_MPI * ret_buf)
+sexp_to_bitmpi (GCRY_SEXP sexp, const char * val, CDK_MPI * ret_buf)
 {
   GCRY_SEXP list = NULL;
   GCRY_MPI m = NULL;
@@ -298,8 +298,8 @@ static int
 sig_to_sexp (GCRY_SEXP * r_sig, cdkPKT_signature * sig)
 {
   GCRY_SEXP sexp = NULL;
-  GCRY_MPI *m = NULL;
-  const char *fmt = NULL;
+  GCRY_MPI * m = NULL;
+  const char * fmt = NULL;
   int ncount = 0;
   int rc = 0;
 
@@ -394,6 +394,15 @@ enc_to_sexp (GCRY_SEXP * r_sexp, cdkPKT_pubkey_enc * enc)
 }
 
 
+static int
+is_unprotected (cdkPKT_secret_key * sk)
+{
+  if (sk->is_protected && !sk->mpi[0])
+    return 0;
+  return 1;
+}
+
+
 int
 cdk_pk_encrypt (cdkPKT_public_key * pk, cdkPKT_pubkey_enc * pke, GCRY_MPI esk)
 {
@@ -431,6 +440,9 @@ cdk_pk_decrypt (cdkPKT_secret_key * sk, cdkPKT_pubkey_enc * pke,
 
   if (!sk || !r_sk || !pke)
     return CDK_Inv_Value;
+
+  if (!is_unprotected (sk))
+    return CDK_Inv_Mode;
   
   rc = skey_to_sexp (&s_skey, sk);
   if (!rc)
@@ -464,6 +476,9 @@ cdk_pk_sign (cdkPKT_secret_key * sk, cdkPKT_signature * sig, const byte * md)
   if (!sk || !sk->pk || !sig || !md)
     return CDK_Inv_Value;
 
+  if (!is_unprotected (sk))
+    return CDK_Inv_Mode;
+  
   if (!KEY_CAN_SIGN (sig->pubkey_algo))
     return CDK_Inv_Algo;
 
@@ -645,7 +660,7 @@ cdk_sk_get_mpi (cdkPKT_secret_key * sk, int idx,
 }
 
 
-static int
+static u16
 checksum_mpi (CDK_MPI m)
 {
   int i;
@@ -653,10 +668,8 @@ checksum_mpi (CDK_MPI m)
 
   if (m)
     {
-      chksum  = (m->bits >> 8) & 0xff;
-      chksum += m->bits & 0xff;
-      for (i = 2; i < m->bytes; i++)
-	chksum += m->data[i];
+      for (i = 0; i < m->bytes + 2; i++)
+        chksum += m->data[i];
     }
   return chksum;
 }
@@ -669,7 +682,7 @@ cdk_seckey_unprotect (cdkPKT_secret_key * sk, char * pw)
   CDK_DEK dek = NULL;
   CDK_MPI a;
   u16 chksum = 0;
-  size_t ndata;
+  size_t ndata, nbits;
   int j, i, dlen, pos = 0, nskey;
   byte * data = NULL;
 
@@ -679,14 +692,21 @@ cdk_seckey_unprotect (cdkPKT_secret_key * sk, char * pw)
   nskey = cdk_pk_get_nskey (sk->pubkey_algo);
   if (sk->is_protected)
     {
-      dek = cdk_passphrase_to_dek (sk->protect.algo, &sk->protect.s2k, 0, pw);
+      dek = cdk_passphrase_to_dek (sk->protect.algo, sk->protect.s2k, 0, pw);
+      if (!dek)
+        return CDK_General_Error;
       hd = gcry_cipher_open (sk->protect.algo,
 			      GCRY_CIPHER_MODE_CFB,
 			      GCRY_CIPHER_SECURE | GCRY_CIPHER_ENABLE_SYNC);
-      if (!hd || !dek)
-	return CDK_Gcry_Error;
+      if (!hd)
+        {
+          cdk_free (dek);
+          return CDK_Gcry_Error;
+        }
       gcry_cipher_setkey (hd, dek->key, dek->keylen);
       gcry_cipher_setiv (hd, sk->protect.iv, sk->protect.ivlen);
+
+      wipemem (dek, sizeof dek);
       cdk_free (dek);
       chksum = 0;
       if (sk->version == 4)
@@ -704,7 +724,10 @@ cdk_seckey_unprotect (cdkPKT_secret_key * sk, char * pw)
 	      chksum = 1;
               dlen = gcry_md_get_algo_dlen (GCRY_MD_SHA1);
 	      if (ndata < dlen)
-		return CDK_Inv_Packet;
+                {
+                  cdk_free (data);
+                  return CDK_Inv_Packet;
+                }
 	      else
 		{
 		  GCRY_MD_HD md = gcry_md_open (GCRY_MD_SHA1, 1);
@@ -728,23 +751,32 @@ cdk_seckey_unprotect (cdkPKT_secret_key * sk, char * pw)
 	    {
 	      for (i = 0; i < nskey; i++)
 		{
-                  a = sk->mpi[i];
-		  a->bits = data[pos] << 8 | data[pos + 1];
-		  a->bytes = (a->bits + 7) / 8 + 2;
-		  for (j = 0; j < a->bytes; j++)
+                  nbits = data[pos] << 8 | data[pos + 1];
+                  ndata = (nbits + 7) / 8;
+                  a = sk->mpi[i] = cdk_scalloc (sizeof *a + ndata + 2);
+                  if (!a)
+                    {
+                      cdk_free (data);
+                      return CDK_Out_Of_Core;
+                    }
+		  a->bits = nbits;
+		  a->bytes = ndata;
+		  for (j = 0; j < ndata + 2; j++)
                     a->data[j] = data[pos++];
 		}
 	    }
+          wipemem (data, sk->enclen);
 	  cdk_free (data);
 	}
       else
 	{
+          chksum = 0;
 	  for (i = 0; i < nskey; i++)
 	    {
-              data = sk->mpi[i]->data;
+              a = sk->mpi[i];
               gcry_cipher_sync (hd);
-	      gcry_cipher_decrypt (hd, data, sk->mpi[i]->bytes, NULL, 0);
-	      chksum += checksum_mpi (sk->mpi[i]);
+	      gcry_cipher_decrypt (hd, a->data + 2, a->bytes, NULL, 0);
+	      chksum += checksum_mpi (a);
 	    }
 	}
       gcry_cipher_close (hd);
@@ -755,6 +787,7 @@ cdk_seckey_unprotect (cdkPKT_secret_key * sk, char * pw)
       for (i = 0; i < nskey; i++)
 	chksum += checksum_mpi (sk->mpi[i]);
     }
+
   if (chksum != sk->csum)
     return CDK_Chksum_Error;
 
