@@ -28,9 +28,9 @@
 #include "main.h"
 #include "packet.h"
 
-#define BUFSIZE 2048
 
-typedef struct {
+typedef struct
+{
   int ctb;
   int type;
   unsigned rfc2440:1;
@@ -49,7 +49,7 @@ stream_getc (CDK_STREAM s)
 
 
 static int
-stream_read (CDK_STREAM s, void *buf, size_t count, size_t *r_nread)
+stream_read (CDK_STREAM s, void * buf, size_t count, size_t * r_nread)
 {
   int nread = cdk_stream_read (s, buf, count);
   if (!nread)
@@ -61,7 +61,7 @@ stream_read (CDK_STREAM s, void *buf, size_t count, size_t *r_nread)
 
 
 u32
-read_32 (CDK_STREAM buf, int *ret_err)
+read_32 (CDK_STREAM buf, int * ret_err)
 {
   u32 u = 0;
   int tmp[4], err = 1;
@@ -76,7 +76,7 @@ read_32 (CDK_STREAM buf, int *ret_err)
 	goto fail;
       if ((tmp[3] = stream_getc (buf)) == EOF)
 	goto fail;
-      u = (tmp[0] << 24) | (tmp[1] << 16) | (tmp[2] << 8) | tmp[3];
+      u = tmp[0] << 24 | tmp[1] << 16 | tmp[2] << 8 | tmp[3];
       err = 0;
     }
  fail:
@@ -87,7 +87,7 @@ read_32 (CDK_STREAM buf, int *ret_err)
 
 
 u16
-read_16 (CDK_STREAM buf, int *ret_err)
+read_16 (CDK_STREAM buf, int * ret_err)
 {
   u16 u = 0;
   int tmp[2], err = 1;
@@ -109,7 +109,7 @@ read_16 (CDK_STREAM buf, int *ret_err)
 
 
 static int
-read_s2k (CDK_STREAM inp, CDK_S2K_HD * s2k)
+read_s2k (CDK_STREAM inp, CDK_S2K_HD s2k)
 {
   size_t nread = 0;
   int rc = 0;
@@ -125,7 +125,7 @@ read_s2k (CDK_STREAM inp, CDK_S2K_HD * s2k)
     return CDK_Inv_Packet;
   s2k->hash_algo = stream_getc (inp);
   if (s2k->mode == 0)
-    ;				/* nothing to do */
+    ; /* nothing to do */
   else if (s2k->mode == 1 || s2k->mode == 3)
     {
       rc = stream_read (inp, s2k->salt, sizeof s2k->salt, &nread);
@@ -170,21 +170,19 @@ read_mpi (CDK_STREAM inp, CDK_MPI * ret_m, int secure)
   m->bytes = nbytes;
   m->bits = nbits;
 
-  /* this is the prefix which encodes the length of the following
-     MPI data. */
+  /* the prefix encodes the length of the MPI data */
   m->data[0] = nbits >> 8;
   m->data[1] = nbits;
   rc = stream_read (inp, m->data + 2, nbytes, &nread);
   if (!rc && nread != nbytes)
     rc = CDK_MPI_Error;
   *ret_m = m;
-
   return rc;
 }
 
 
 static int
-read_pkt_length (CDK_STREAM inp, int *ret_partial)
+read_pkt_length (CDK_STREAM inp, int * ret_partial)
 {
   int c1 = 0, c2 = 0;
   size_t pktlen = 0;
@@ -193,12 +191,13 @@ read_pkt_length (CDK_STREAM inp, int *ret_partial)
   if ((c1 = stream_getc (inp)) == EOF)
     return EOF;
   if (c1 < 224 || c1 == 255)
-    *ret_partial = 0;		/* end of partial data */
+    *ret_partial = 0; /* end of partial data */
   if (c1 < 192)
     pktlen = c1;
   else if (c1 >= 192 && c1 <= 223)
     {
-      if ((c2 = stream_getc (inp)) == EOF)
+      c2 = stream_getc (inp);
+      if (c2 == EOF)
 	return EOF;
       pktlen = ((c1 - 192) << 8) + c2 + 192;
     }
@@ -218,8 +217,8 @@ static int
 read_encrypted (CDK_STREAM inp, size_t pktlen, cdkPKT_encrypted * enc,
 		int partial, int mdc)
 {
-  byte buf[BUFSIZE];
-  size_t nread = 0;
+  byte buf[8192];
+  size_t nread = 0, buflen;
   int rc = 0, version;
 
   if (!inp || !enc)
@@ -242,20 +241,20 @@ read_encrypted (CDK_STREAM inp, size_t pktlen, cdkPKT_encrypted * enc,
     }
   while (!cdk_stream_eof (inp))
     {
-      rc = stream_read (inp, buf, sizeof buf-1, &nread);
+      buflen = partial? pktlen : sizeof buf-1;
+      rc = stream_read (inp, buf, buflen, &nread);
       if (rc)
         goto leave;
       pktlen -= nread;
       cdk_stream_write (enc->buf, buf, nread);
-      if (!partial)
-	break;
-      else
+      if (partial)
 	{
 	  pktlen = read_pkt_length (inp, &partial);
 	  if (pktlen == EOF)
 	    return CDK_Inv_Packet;
 	}
     }
+  wipemem (buf, sizeof buf);
   if (pktlen)
     return CDK_Inv_Packet;
   cdk_stream_seek (enc->buf, 0);
@@ -269,6 +268,7 @@ leave:
 static int
 read_symkey_enc (CDK_STREAM inp, size_t pktlen, cdkPKT_symkey_enc * ske)
 {
+  CDK_S2K_HD s2k;
   size_t nread = 0, minlen = 0;
   int rc = 0;
 
@@ -282,24 +282,28 @@ read_symkey_enc (CDK_STREAM inp, size_t pktlen, cdkPKT_symkey_enc * ske)
   if (ske->version != 4)
     return CDK_Inv_Packet;
 
+  s2k = ske->s2k = cdk_calloc (1, sizeof *ske->s2k);
+  if (!ske->s2k)
+    return CDK_Out_Of_Core;
+  
   ske->cipher_algo = stream_getc (inp);
-  ske->s2k.mode = stream_getc (inp);
-  switch (ske->s2k.mode)
+  s2k->mode = stream_getc (inp);
+  switch (s2k->mode)
     {
     case 0: minlen = 0; break;
     case 1: minlen = 8; break;
     case 3: minlen = 9; break;
     }
-  ske->s2k.hash_algo = stream_getc (inp);
-  if (ske->s2k.mode == 0)
+  s2k->hash_algo = stream_getc (inp);
+  if (s2k->mode == 0)
     ; /* nothing to do */
-  if (ske->s2k.mode == 1 || ske->s2k.mode == 3)
+  if (s2k->mode == 1 || s2k->mode == 3)
     {
-      rc = stream_read (inp, ske->s2k.salt, DIM (ske->s2k.salt), &nread);
-      if (!rc && nread != DIM (ske->s2k.salt))
+      rc = stream_read (inp, s2k->salt, DIM (s2k->salt), &nread);
+      if (!rc && nread != DIM (s2k->salt))
 	return CDK_Inv_Packet;
-      if (!rc && ske->s2k.mode == 3)
-	ske->s2k.count = stream_getc (inp);
+      if (!rc && s2k->mode == 3)
+	s2k->count = stream_getc (inp);
     }
   else
     return CDK_Inv_Packet;
@@ -317,7 +321,7 @@ read_symkey_enc (CDK_STREAM inp, size_t pktlen, cdkPKT_symkey_enc * ske)
 
 
 static int
-read_pubkey_enc (CDK_STREAM inp, cdkPKT_pubkey_enc * pke)
+read_pubkey_enc (CDK_STREAM inp, size_t pktlen, cdkPKT_pubkey_enc * pke)
 {
   int rc = 0;
   int i, nenc = 0;
@@ -328,13 +332,15 @@ read_pubkey_enc (CDK_STREAM inp, cdkPKT_pubkey_enc * pke)
   if (DEBUG_PKT)
     _cdk_log_debug ("** read public key encrypted packet\n");
 
+  if (pktlen < 10)
+    return CDK_Inv_Packet;
   pke->version = stream_getc (inp);
   if (pke->version < 2 || pke->version > 3)
     return CDK_Inv_Packet;
   pke->keyid[0] = read_32 (inp, NULL);
   pke->keyid[1] = read_32 (inp, NULL);
   if (!pke->keyid[0] && !pke->keyid[1])
-    pke->throw_keyid = 1;	/* RFC2440 "speculative" keyID */
+    pke->throw_keyid = 1; /* RFC2440 "speculative" keyID */
   pke->pubkey_algo = stream_getc (inp);
   nenc = cdk_pk_get_nenc (pke->pubkey_algo);
   for (i = 0; i < nenc; i++)
@@ -406,9 +412,9 @@ read_public_key (CDK_STREAM inp, cdkPKT_public_key * pk)
   pk->version = stream_getc (inp);
   if (pk->version < 2 || pk->version > 4)
       return CDK_Inv_Packet_Ver;
+  pk->timestamp = read_32 (inp, &rc);
   if (pk->version < 4)
     pk->expiredate = read_16 (inp, &rc);
-  pk->timestamp = read_32 (inp, &rc);
   pk->pubkey_algo = stream_getc (inp);
   for (i = 0; i < cdk_pk_get_npkey (pk->pubkey_algo); i++)
     {
@@ -454,7 +460,10 @@ read_secret_key (CDK_STREAM inp, size_t pktlen, cdkPKT_secret_key * sk)
     {
       sk->protect.sha1chk = (sk->s2k_usage == 254);
       sk->protect.algo = stream_getc (inp);
-      rc = read_s2k (inp, &sk->protect.s2k);
+      sk->protect.s2k = cdk_calloc (1, sizeof *sk->protect.s2k);
+      if (!sk->protect.s2k)
+        return CDK_Out_Of_Core;
+      rc = read_s2k (inp, sk->protect.s2k);
       if (rc)
 	return rc;
       blklen = gcry_cipher_get_algo_blklen (sk->protect.algo);
@@ -514,11 +523,7 @@ read_secret_key (CDK_STREAM inp, size_t pktlen, cdkPKT_secret_key * sk)
 	return CDK_Inv_Packet;
       nskey = cdk_pk_get_nskey (sk->pk->pubkey_algo);
       for (i = 0; i < nskey; i++)
-	{
-	  sk->mpi[i] = cdk_salloc (sizeof **sk->mpi + MAX_MPI_BYTES);
-	  if (!sk->mpi[i])
-	    return CDK_Out_Of_Core;
-	}
+        sk->mpi[i] = NULL;
       sk->is_protected = 1;
     }
   sk->is_primary = 1;
@@ -621,7 +626,7 @@ read_user_id (CDK_STREAM inp, size_t pktlen, cdkPKT_user_id * user_id)
 
 
 static int
-read_subpkt (CDK_STREAM inp, CDK_SUBPKT * r_ctx, size_t *r_nbytes)
+read_subpkt (CDK_STREAM inp, CDK_SUBPKT * r_ctx, size_t * r_nbytes)
 {
   byte c, c1;
   size_t size = 0, nread, n = 0;
@@ -683,7 +688,7 @@ read_subpkt (CDK_STREAM inp, CDK_SUBPKT * r_ctx, size_t *r_nbytes)
 
 
 static int
-read_onepass_sig (CDK_STREAM inp, cdkPKT_onepass_sig * sig)
+read_onepass_sig (CDK_STREAM inp, size_t pktlen, cdkPKT_onepass_sig * sig)
 {
   if (!inp || !sig)
     return CDK_Inv_Value;
@@ -691,6 +696,8 @@ read_onepass_sig (CDK_STREAM inp, cdkPKT_onepass_sig * sig)
   if (DEBUG_PKT)
     _cdk_log_debug ("** read one pass signature packet\n");
 
+  if (pktlen < 13)
+    return CDK_Inv_Packet;
   sig->version = stream_getc (inp);
   if (sig->version != 3)
     return CDK_Inv_Packet_Ver;
@@ -705,7 +712,7 @@ read_onepass_sig (CDK_STREAM inp, cdkPKT_onepass_sig * sig)
 
 
 static int
-read_signature (CDK_STREAM inp, cdkPKT_signature * sig)
+read_signature (CDK_STREAM inp, size_t pktlen, cdkPKT_signature * sig)
 {
   struct cdk_subpkt_s *p = NULL;
   size_t nbytes;
@@ -718,6 +725,8 @@ read_signature (CDK_STREAM inp, cdkPKT_signature * sig)
   if (DEBUG_PKT)
     _cdk_log_debug ("** read signature packet\n");
 
+  if (pktlen < 10)
+    return CDK_Inv_Packet;
   sig->version = stream_getc (inp);
   if (sig->version < 2 || sig->version > 4)
       return CDK_Inv_Packet_Ver;
@@ -794,7 +803,7 @@ read_signature (CDK_STREAM inp, cdkPKT_signature * sig)
 	  else if (p->type == SIGSUBPKT_SIG_EXPIRE)
 	    {
 	      sig->expiredate = buffer_to_u32 (p->d);
-	      if (sig->expiredate > 0 && sig->expiredate < make_timestamp ())
+	      if (sig->expiredate > 0 && sig->expiredate < _cdk_timestamp ())
 		sig->flags.expired = 1;
 	    }
 	  else if (p->type == SIGSUBPKT_POLICY)
@@ -1021,14 +1030,14 @@ cdk_pkt_parse (CDK_STREAM inp, CDK_PACKET * pkt)
       pkt->pkt.onepass_sig = cdk_calloc (1, sizeof *pkt->pkt.onepass_sig);
       if (!pkt->pkt.onepass_sig)
 	return CDK_Out_Of_Core;
-      rc = read_onepass_sig (inp, pkt->pkt.onepass_sig);
+      rc = read_onepass_sig (inp, pkth.len, pkt->pkt.onepass_sig);
       break;
 
     case CDK_PKT_SIGNATURE:
       pkt->pkt.signature = cdk_calloc (1, sizeof *pkt->pkt.signature);
       if (!pkt->pkt.signature)
 	return CDK_Out_Of_Core;
-      rc = read_signature (inp, pkt->pkt.signature);
+      rc = read_signature (inp, pkth.len, pkt->pkt.signature);
       break;
 
     case CDK_PKT_ENCRYPTED_MDC:
@@ -1052,7 +1061,7 @@ cdk_pkt_parse (CDK_STREAM inp, CDK_PACKET * pkt)
       pkt->pkt.pubkey_enc = cdk_calloc (1, sizeof *pkt->pkt.pubkey_enc);
       if (!pkt->pkt.pubkey_enc)
 	return CDK_Out_Of_Core;
-      rc = read_pubkey_enc (inp, pkt->pkt.pubkey_enc);
+      rc = read_pubkey_enc (inp, pkth.len, pkt->pkt.pubkey_enc);
       break;
 
     case CDK_PKT_COMPRESSED:
