@@ -29,7 +29,7 @@
 #include "main.h"
 #include "packet.h"
 
-#define gcry_md_get_asnoid( algo, asnbuf, asnlen ) \
+#define gcry_md_get_asnoid(algo, asnbuf, asnlen) \
 gcry_md_algo_info( (algo), GCRYCTL_GET_ASNOID, (asnbuf), (asnlen) )
 
 /* We encode the MD in this way:
@@ -43,7 +43,7 @@ do_encode_md (byte ** r_frame, size_t * r_flen, const byte * md, int algo,
 	      size_t len, unsigned nbits, const byte * asn, size_t asnlen)
 {
   int i, nframe = (nbits + 7) / 8;
-  byte *frame = NULL;
+  byte * frame = NULL;
   size_t n = 0;
 
   if (!asn || !md)
@@ -156,7 +156,6 @@ _cdk_pkcs1_sesskey (GCRY_MPI * esk, CDK_DEK dek, int nbits)
   cdk_free (frame);
   if (!rc)
     *esk = a;
-
   return rc;
 }
 
@@ -273,10 +272,54 @@ _cdk_pkcs1_digest (byte ** r_md, size_t * r_mdlen, int pk_algo,
 }
 
 
+static char *
+passphrase_prompt (cdkPKT_secret_key * sk)
+{
+  char * p;
+  int bits = cdk_pk_get_nbits (sk->pk), pk_algo = sk->pubkey_algo;
+  u32 keyid = cdk_pk_get_keyid (sk->pk, NULL);
+  const char *algo = "???";
+  
+  p = cdk_calloc (1, 512);
+  if (!p)
+    return NULL;
+
+  if (is_RSA (pk_algo))
+    algo = "RSA";
+  else if (is_ELG (pk_algo))
+    algo = "ELG";
+  else if (is_DSA (pk_algo))
+    algo = "DSA";
+  
+  snprintf (p, 511, "%d-bit %s key, ID %08lX\nEnter Passphrase: ",
+            bits, algo, keyid);
+  
+  return p;
+}
+
+
+int
+_cdk_seckey_unprotect2 (cdkPKT_secret_key * sk)
+{
+  char * pw = NULL, * p = NULL;
+  int rc = 0;
+  
+  if (sk->is_protected)
+    {
+      p = passphrase_prompt (sk);
+      pw = _cdk_passphrase_get (p);
+      if (pw)
+        rc = cdk_seckey_unprotect (sk, pw);
+      _cdk_passphrase_free (pw);
+      cdk_free (p);
+    }
+  return rc;
+}
+
+
 CDK_DEK
 cdk_pkcs1_to_dek (cdkPKT_pubkey_enc * enc, cdkPKT_secret_key * sk)
 {
-  char * pw = NULL;
   CDK_DEK dek = NULL;
   GCRY_MPI skey = NULL;
   int rc = 0;
@@ -285,19 +328,11 @@ cdk_pkcs1_to_dek (cdkPKT_pubkey_enc * enc, cdkPKT_secret_key * sk)
       return NULL;
   
   if (sk->is_protected)
-    {
-      _cdk_log_debug ("%d-bit key, ID %08lX\n", cdk_pk_get_nbits (sk->pk),
-                      cdk_sk_get_keyid (sk, NULL));
-      pw = _cdk_passphrase_get ("Enter Passphrase: ");
-      if (pw)
-        rc = cdk_seckey_unprotect (sk, pw);
-      _cdk_passphrase_free (pw);
-    }
+    rc = _cdk_seckey_unprotect2 (sk);
   if (!rc)
     rc = cdk_pk_decrypt (sk, enc, &skey);
   if (!rc)
-    dek = pkcs1_decode(skey);
-  
+    dek = pkcs1_decode (skey);
   gcry_mpi_release (skey);
   return dek;
 }
@@ -344,7 +379,7 @@ cdk_dek_new (CDK_DEK * r_dek, int algo)
 
 
 CDK_DEK
-cdk_passphrase_to_dek (int cipher_algo, CDK_S2K_HD * s2k, int mode, char * pw)
+cdk_passphrase_to_dek (int cipher_algo, CDK_S2K_HD s2k, int mode, char * pw)
 {
   CDK_DEK dek;
 
@@ -362,7 +397,7 @@ cdk_passphrase_to_dek (int cipher_algo, CDK_S2K_HD * s2k, int mode, char * pw)
 
 
 int
-cdk_hash_passphrase (CDK_DEK dek, char *pw, CDK_S2K_HD * s2k, int create)
+cdk_hash_passphrase (CDK_DEK dek, char *pw, CDK_S2K_HD s2k, int create)
 {
   GCRY_MD_HD md;
   int pass, i;
@@ -433,3 +468,31 @@ cdk_hash_passphrase (CDK_DEK dek, char *pw, CDK_S2K_HD * s2k, int create)
   gcry_md_close (md);
   return 0;
 }
+
+
+int
+cdk_s2k_new (CDK_S2K_HD * ret_s2k, int mode, int algo, const byte * salt)
+{
+  CDK_S2K_HD s2k;
+  int rc;
+
+  if (mode < 0 || mode > 3)
+    return CDK_Inv_Value;
+  
+  rc = _cdk_md_test_algo (algo);
+  if (rc)
+    return rc;
+  
+  s2k = cdk_calloc (1, sizeof *s2k);
+  if (!s2k)
+    return CDK_Out_Of_Core;
+  s2k->mode = mode;
+  s2k->hash_algo = algo;
+  if (salt)
+    memcpy (s2k->salt, salt, 8);
+
+  if (ret_s2k)
+    *ret_s2k = s2k;
+
+  return 0;
+}
