@@ -51,7 +51,7 @@ hash_mpi_array (cdkPKT_public_key * pk, GCRY_MD_HD md, int v4)
   
 
 void
-_cdk_hash_pubkey (cdkPKT_public_key * pk, GCRY_MD_HD md)
+_cdk_hash_pubkey (cdkPKT_public_key * pk, GCRY_MD_HD md, int use_fpr)
 {
   int i, npkey;
   u16 n;
@@ -61,7 +61,7 @@ _cdk_hash_pubkey (cdkPKT_public_key * pk, GCRY_MD_HD md)
     return;
 
   npkey = cdk_pk_get_npkey (pk->pubkey_algo);
-  if (pk->version < 4 && is_RSA (pk->pubkey_algo))
+  if (use_fpr && pk->version < 4 && is_RSA (pk->pubkey_algo))
     {
       hash_mpi_array (pk, md, 0);
       return;
@@ -70,14 +70,14 @@ _cdk_hash_pubkey (cdkPKT_public_key * pk, GCRY_MD_HD md)
   for (i = 0; i < npkey; i++)
     {
       n += pk->mpi[i]->bytes;
-      if (pk->version == 4)
-	n += 2; /* prefix */
+      n += 2;
     }
+  
   gcry_md_putc (md, 0x99);
   gcry_md_putc (md, n >> 8);
   gcry_md_putc (md, n);
-  gcry_md_putc (md, 4);
-
+  gcry_md_putc (md, pk->version);
+  
   buf[0] = pk->timestamp >> 24;
   buf[1] = pk->timestamp >> 16;
   buf[2] = pk->timestamp >> 8;
@@ -91,7 +91,7 @@ _cdk_hash_pubkey (cdkPKT_public_key * pk, GCRY_MD_HD md)
       gcry_md_putc (md, a);
     }
   gcry_md_putc (md, pk->pubkey_algo);
-  hash_mpi_array (pk, md, pk->version == 4);
+  hash_mpi_array (pk, md, 1);
 }
 
 
@@ -204,7 +204,7 @@ _cdk_signature_check (cdkPKT_public_key * pk, cdkPKT_signature * sig,
                       GCRY_MD_HD digest, int * r_expired)
 {
   byte md[24];
-  time_t cur_time = make_timestamp ();
+  time_t cur_time = _cdk_timestamp ();
   int digest_algo;
   int rc;
 
@@ -266,7 +266,7 @@ check_key_signature (CDK_KEYDB_HD hd, CDK_KBNODE knode, CDK_KBNODE snode)
 
   if (sig->sig_class == 0x20) /* key revocation */
     {
-      _cdk_hash_pubkey (pk, md);
+      _cdk_hash_pubkey (pk, md, 0);
       rc = _cdk_signature_check (pk, sig, md, &is_expired);
     }
   else if (sig->sig_class == 0x28) /* subkey revocation */
@@ -274,8 +274,8 @@ check_key_signature (CDK_KEYDB_HD hd, CDK_KBNODE knode, CDK_KBNODE snode)
       node = cdk_kbnode_find_prev (knode, snode, CDK_PKT_PUBLIC_SUBKEY);
       if (!node)
 	return CDK_Error_No_Key; /* no subkey for subkey revocation packet */
-      _cdk_hash_pubkey (pk, md);
-      _cdk_hash_pubkey (node->pkt->pkt.public_key, md);
+      _cdk_hash_pubkey (pk, md, 0);
+      _cdk_hash_pubkey (node->pkt->pkt.public_key, md, 0);
       rc = _cdk_signature_check (pk, sig, md, &is_expired);
     }
   else if (sig->sig_class == 0x18) /* key binding */
@@ -283,13 +283,13 @@ check_key_signature (CDK_KEYDB_HD hd, CDK_KBNODE knode, CDK_KBNODE snode)
       node = cdk_kbnode_find_prev (knode, snode, CDK_PKT_PUBLIC_SUBKEY);
       if (!node)
 	return CDK_Error_No_Key; /* no subkey for subkey binding packet */
-      _cdk_hash_pubkey (pk, md);
-      _cdk_hash_pubkey (node->pkt->pkt.public_key, md);
+      _cdk_hash_pubkey (pk, md, 0);
+      _cdk_hash_pubkey (node->pkt->pkt.public_key, md, 0);
       rc = _cdk_signature_check (pk, sig, md, &is_expired);
     }
   else if (sig->sig_class == 0x1f) /* direct key signature */
     {
-      _cdk_hash_pubkey (pk, md);
+      _cdk_hash_pubkey (pk, md, 0);
       rc = _cdk_signature_check (pk, sig, md, &is_expired);
     }
   else /* all other classes */
@@ -297,7 +297,7 @@ check_key_signature (CDK_KEYDB_HD hd, CDK_KBNODE knode, CDK_KBNODE snode)
       node = cdk_kbnode_find_prev (knode, snode, CDK_PKT_USER_ID);
       if (!node)
 	return CDK_Error_No_Key; /* no user ID for key signature packet */
-      _cdk_hash_pubkey (pk, md);
+      _cdk_hash_pubkey (pk, md, 0);
       hash_userid (node->pkt->pkt.user_id, sig->version, md);
       if (pk->keyid[0] == sig->keyid[0] && pk->keyid[1] == sig->keyid[1])
 	rc = _cdk_signature_check (pk, sig, md, &is_expired);
