@@ -25,8 +25,10 @@
 #include <sys/stat.h>
 #include <string.h>
 #include <stdlib.h>
-#include <unistd.h>
 #include <errno.h>
+#ifdef HAVE_UNISTD_H
+# include <unistd.h>
+#endif
 
 #include "opencdk.h"
 #include "filters.h"
@@ -47,7 +49,7 @@ struct stream_filter_table_s filter_table[] = {
 };
 
 static int stream_filter_walk_write (CDK_STREAM s);
-static int stream_cache_flush (CDK_STREAM s, FILE *fp);
+static int stream_cache_flush (CDK_STREAM s, FILE * fp);
 
 
 #ifndef HAVE_MKSTEMP
@@ -69,7 +71,7 @@ mkstemp (char * template)
 }
 #endif
 
-static char*
+static char *
 mk_tmpnam (void)
 {
   static char template[11];
@@ -88,7 +90,7 @@ mk_tmpnam (void)
  * @ret_s: The new STREAM object
  **/
 int
-cdk_stream_open (const char *file, CDK_STREAM *ret_s)
+cdk_stream_open (const char * file, CDK_STREAM * ret_s)
 {
   CDK_STREAM s;
 
@@ -125,7 +127,7 @@ cdk_stream_open (const char *file, CDK_STREAM *ret_s)
  * @ret_s: The new STREAM object
  **/
 int
-cdk_stream_new (const char *file, CDK_STREAM *ret_s)
+cdk_stream_new (const char * file, CDK_STREAM * ret_s)
 {
   CDK_STREAM s;
 
@@ -188,7 +190,7 @@ cdk_stream_tmp (void)
 
 
 CDK_STREAM
-cdk_stream_tmp_from_mem (const byte *buf, size_t count)
+cdk_stream_tmp_from_mem (const byte * buf, size_t count)
 {
   CDK_STREAM s;
   int nwritten;
@@ -210,7 +212,7 @@ cdk_stream_tmp_from_mem (const byte *buf, size_t count)
   
 
 CDK_STREAM
-cdk_stream_fpopen (FILE *fp, unsigned write_mode)
+cdk_stream_fpopen (FILE * fp, unsigned write_mode)
 {
   CDK_STREAM s;
 
@@ -226,6 +228,35 @@ cdk_stream_fpopen (FILE *fp, unsigned write_mode)
 }
 
 
+int
+_cdk_stream_append (const char * file, CDK_STREAM * ret_s)
+{
+  CDK_STREAM s;
+  FILE * fp;
+  int rc;
+
+  if (ret_s)
+    *ret_s = NULL;
+  
+  rc = cdk_stream_open (file, &s);
+  if (rc)
+    return rc;
+  fp = fopen (file, "a+b");
+  if (!fp)
+    {
+      cdk_stream_close (s);
+      return CDK_File_Error;
+    }
+  fclose (s->fp);
+  s->fp = fp;
+  s->flags.write = 1;
+  if (ret_s)
+    *ret_s = s;
+  return 0;
+}
+
+      
+
 int
 cdk_stream_control (CDK_STREAM s, int ctl, int val)
 {
@@ -245,6 +276,37 @@ cdk_stream_control (CDK_STREAM s, int ctl, int val)
 }
 
 
+int
+cdk_stream_flush (CDK_STREAM s)
+{
+  int rc = 0;
+  
+  if (!s)
+    return CDK_Inv_Value;
+
+  if (!s->flags.filtrated)
+    {
+      rc = fseek (s->fp, 0, SEEK_SET);
+      if (!rc)
+        rc = fflush (s->fp);
+      if (rc)
+        rc = CDK_File_Error;
+      if (!rc)
+        rc = stream_filter_walk_write (s);
+      s->flags.filtrated = 1;
+    }
+  return rc;
+}
+
+
+void
+cdk_stream_tmp_set_mode (CDK_STREAM s, int val)
+{
+  if (s && s->flags.temp)
+    s->fmode = val;
+}
+
+
 /**
  * cdk_stream_close: Close a stream and flush all buffers.
  * @s: The STREAM object.
@@ -258,22 +320,13 @@ cdk_stream_control (CDK_STREAM s, int ctl, int val)
 int
 cdk_stream_close (CDK_STREAM s)
 {
-  struct stream_filter_s *f, *f2;
-  int rc = 0, rc2 = 0;
+  struct stream_filter_s * f, * f2;
+  int rc = 0;
   
   if (s)
     {
       if (!s->flags.filtrated)
-        {
-          rc2 = fseek (s->fp, 0, SEEK_SET);
-          if (!rc2)
-            rc2 = fflush (s->fp);
-          if (rc2)
-            rc2 = CDK_File_Error;
-          if (!rc2)
-            rc2 = stream_filter_walk_write (s);
-          s->flags.filtrated = 1;
-        }
+        rc = cdk_stream_flush (s);
       else if (s->fname || s->flags.temp)
         {
           rc = fclose (s->fp);
@@ -296,8 +349,6 @@ cdk_stream_close (CDK_STREAM s)
         }
       cdk_free (s);
     }
-  if (!rc && rc2)
-    rc = rc2;
   return rc;
 }
 
@@ -322,6 +373,13 @@ _cdk_stream_get_fname (CDK_STREAM s)
 }
 
 
+FILE *
+_cdk_stream_get_fp (CDK_STREAM s)
+{
+  return s->fp;
+}
+  
+
 /**
  * cdk_stream_get_length: Return the length of the associated file handle.
  * @s: The STREAM object.
@@ -347,7 +405,7 @@ cdk_stream_get_length (CDK_STREAM s)
 static struct stream_filter_s*
 filter_add (CDK_STREAM s)
 {
-  struct stream_filter_s *f;
+  struct stream_filter_s * f;
 
   f = cdk_calloc (1, sizeof *f);
   if (!f)
@@ -362,7 +420,7 @@ filter_add (CDK_STREAM s)
 static struct stream_filter_s *
 filter_search (CDK_STREAM s, FILTER_FUNC fnc)
 {
-  struct stream_filter_s *f;
+  struct stream_filter_s * f;
 
   for (f = s->filters; f; f = f->next)
     {
@@ -376,7 +434,7 @@ filter_search (CDK_STREAM s, FILTER_FUNC fnc)
 struct stream_filter_s *
 stream_filter_add (CDK_STREAM s, FILTER_FUNC fnc, int type)
 {
-  struct stream_filter_s *flt;
+  struct stream_filter_s * flt;
 
   s->flags.filtrated = 0;
   flt = filter_search (s, fnc);
@@ -407,10 +465,10 @@ stream_filter_add (CDK_STREAM s, FILTER_FUNC fnc, int type)
 static int
 stream_mode (CDK_STREAM s)
 {
-  if (!s->flags.temp)
-    return s->flags.write;
+  if (s->flags.temp)
+    return s->fmode;
   
-  return 0; /* in the temp mode only decoding is allowed (fixme?) */
+  return s->flags.write;
 }
 
 
@@ -423,7 +481,7 @@ stream_mode (CDK_STREAM s)
 int
 cdk_stream_filter_disable (CDK_STREAM s, int type)
 {
-  struct stream_filter_s *f;
+  struct stream_filter_s * f;
   FILTER_FUNC fnc;
   int idx;
 
@@ -438,6 +496,20 @@ cdk_stream_filter_disable (CDK_STREAM s, int type)
 }
 
 
+static int
+stream_fp_replace (CDK_STREAM s, FILE ** tmp)
+{
+  int rc;
+  
+  rc = fclose (s->fp);
+  if (rc)
+    return CDK_File_Error;
+  s->fp = *tmp;
+  *tmp = NULL;
+  return 0;
+}
+
+
 /* This function is a little more complicated because we can't use tmpfile()
    because we have to rename the final file. So what are we doing here:
    In stream_new a tempfile was created. Now we create a second tempfile and
@@ -448,7 +520,7 @@ cdk_stream_filter_disable (CDK_STREAM s, int type)
 static int
 stream_filter_walk_write (CDK_STREAM s)
 {
-  struct stream_filter_s *f;
+  struct stream_filter_s * f;
   int rc = 0;
   
   if (s->flags.filtrated)
@@ -472,23 +544,18 @@ stream_filter_walk_write (CDK_STREAM s)
         {
           rc = stream_cache_flush (s, f->tmp);
           if (rc)
-            return rc;
+            break;
         }
       rc = f->fnc (f->opaque, s->fp, f->tmp);
       if (rc)
         {
           fclose (f->tmp);
           unlink (s->tmpn);
-          return rc;
-        }
-      rc = fclose (s->fp);
-      if (rc)
-        {
-          rc = CDK_File_Error;
           break;
         }
-      s->fp = f->tmp;
-      f->tmp = NULL;
+      rc = stream_fp_replace (s, &f->tmp);
+      if (rc)
+        break;
       rc = fseek (s->fp, 0, SEEK_SET);
       if (rc)
         {
@@ -514,11 +581,13 @@ stream_filter_walk_write (CDK_STREAM s)
     }
   if (!rc)
     {
-      fclose (s->fp);
       if (!s->flags.temp)
-        rc = rename (s->tname, s->fname);
-      if (rc)
-        rc = CDK_File_Error;
+        {
+          fclose (s->fp);
+          rc = rename (s->tname, s->fname);
+          if (rc)
+            rc = CDK_File_Error;
+        }
     }
   return rc;
 }
@@ -532,7 +601,7 @@ stream_filter_walk_write (CDK_STREAM s)
 static int
 stream_filter_walk_read (CDK_STREAM s)
 {
-  struct stream_filter_s *f;
+  struct stream_filter_s * f;
   int rc = 0;
   
   if (s->flags.filtrated)
@@ -556,14 +625,9 @@ stream_filter_walk_read (CDK_STREAM s)
          the contents were not altered in any way. */
       if (!f->flags.rdonly)
         {
-          rc = fclose (s->fp);
+          rc = stream_fp_replace (s, &f->tmp);
           if (rc)
-            {
-              rc = CDK_File_Error;
-              break;
-            }
-          s->fp = f->tmp;
-          f->tmp = NULL;
+            break;
         }
       else
         {
@@ -583,10 +647,10 @@ stream_filter_walk_read (CDK_STREAM s)
 }
 
 
-void*
+void *
 _cdk_stream_get_opaque (CDK_STREAM s, int id)
 {
-  struct stream_filter_s *f;
+  struct stream_filter_s * f;
   
   for (f = s->filters; f; f = f->next)
     {
@@ -608,7 +672,7 @@ _cdk_stream_get_opaque (CDK_STREAM s, int id)
  * need to add the filters in reserved order.
  **/
 int
-cdk_stream_read (CDK_STREAM s, void *buf, size_t count)
+cdk_stream_read (CDK_STREAM s, void * buf, size_t count)
 {
   int nread;
   int rc;
@@ -659,7 +723,7 @@ cdk_stream_getc (CDK_STREAM s)
  * partial flushing.
  **/
 int
-cdk_stream_write (CDK_STREAM s, const void *buf, size_t count)
+cdk_stream_write (CDK_STREAM s, const void * buf, size_t count)
 {
   int nwritten;
 
@@ -718,7 +782,7 @@ cdk_stream_seek (CDK_STREAM s, long offset)
 int
 cdk_stream_set_armor_flag (CDK_STREAM s, int type)
 {
-  struct stream_filter_s *flt;
+  struct stream_filter_s * flt;
   FILTER_FUNC fnc;
   int idx = fARMOR;
 
@@ -732,9 +796,9 @@ cdk_stream_set_armor_flag (CDK_STREAM s, int type)
 
 
 int
-cdk_stream_set_literal_flag (CDK_STREAM s, int mode, const char *fname)
+cdk_stream_set_literal_flag (CDK_STREAM s, int mode, const char * fname)
 {
-  struct stream_filter_s *flt;
+  struct stream_filter_s * flt;
   FILTER_FUNC fnc;
   int idx = fPLAINTEXT;
 
@@ -743,7 +807,7 @@ cdk_stream_set_literal_flag (CDK_STREAM s, int mode, const char *fname)
   if (!flt)
     return CDK_Out_Of_Core;
   flt->u.pfx.mode = mode;
-  flt->u.pfx.filename = cdk_strdup (fname);
+  flt->u.pfx.filename = fname? cdk_strdup (fname) : NULL;
   return 0;
 }
 
@@ -751,7 +815,7 @@ cdk_stream_set_literal_flag (CDK_STREAM s, int mode, const char *fname)
 int
 cdk_stream_set_cipher_flag (CDK_STREAM s, CDK_DEK dek, int use_mdc)
 {
-  struct stream_filter_s *flt;
+  struct stream_filter_s * flt;
   FILTER_FUNC fnc;
   int idx = fCIPHER;
 
@@ -769,7 +833,7 @@ cdk_stream_set_cipher_flag (CDK_STREAM s, CDK_DEK dek, int use_mdc)
 int
 cdk_stream_set_compress_flag (CDK_STREAM s, int algo)
 {
-  struct stream_filter_s *flt;
+  struct stream_filter_s * flt;
   FILTER_FUNC fnc;
   int idx = fCOMPRESS;
 
@@ -783,9 +847,9 @@ cdk_stream_set_compress_flag (CDK_STREAM s, int algo)
 
 
 int
-cdk_stream_set_text_flag (CDK_STREAM s, const char *lf)
+cdk_stream_set_text_flag (CDK_STREAM s, const char * lf)
 {
-  struct stream_filter_s *flt;
+  struct stream_filter_s * flt;
   FILTER_FUNC fnc;
   int idx = fTEXT;
 
@@ -801,7 +865,7 @@ cdk_stream_set_text_flag (CDK_STREAM s, const char *lf)
 int
 cdk_stream_set_hash_flag (CDK_STREAM s, int algo)
 {
-  struct stream_filter_s *flt;
+  struct stream_filter_s * flt;
   FILTER_FUNC fnc;
   int idx = fHASH;
 
@@ -829,7 +893,7 @@ cdk_stream_set_cache (CDK_STREAM s, int val)
 
 
 static int
-stream_cache_flush (CDK_STREAM s, FILE *fp)
+stream_cache_flush (CDK_STREAM s, FILE * fp)
 {
   int nwritten;
 
@@ -864,6 +928,7 @@ cdk_stream_kick_off (CDK_STREAM inp, CDK_STREAM out)
       if (nwritten == EOF)
         rc = CDK_File_Error;
     }
+  wipemem (buf, sizeof buf);
   return rc;
 }
 
