@@ -30,14 +30,19 @@
 #include "filters.h"
 #include "packet.h"
 
-struct {
+
+#define GCRY_MD_MD2 5 /* dummy to detect it */
+
+struct
+{
   const char *name;
   int algo;
-} digest_table[] = {
+} digest_table[] =
+{
   {"MD5",       GCRY_MD_MD5},
   {"SHA1",      GCRY_MD_SHA1},
   {"RIPEMD160", GCRY_MD_RMD160},
-  {"MD2",       0}, /* no support for MD2 */
+  {"MD2",       GCRY_MD_MD2},
   {"TIGER192",  GCRY_MD_TIGER},
   {NULL, 0}
 };
@@ -196,11 +201,19 @@ cdk_file_verify_clearsign (CDK_HD hd, const char * file, const char * output)
           for (i = 0; (s = digest_table[i].name); i++)
             {
               if (!strcmp (buf + 6, s))
-                digest_algo = digest_table[i].algo;
+                {
+                  digest_algo = digest_table[i].algo;
+                  break;
+                }
             }
         }
     }
 
+  if (digest_algo == GCRY_MD_MD2)
+    {
+      rc = CDK_Inv_Algo;
+      goto leave;
+    }
   if (!digest_algo)
     digest_algo = GCRY_MD_MD5;
   md = gcry_md_open (digest_algo, 0);
@@ -259,6 +272,7 @@ cdk_file_verify_clearsign (CDK_HD hd, const char * file, const char * output)
         }
       cdk_stream_write (tmp, buf, nbytes);
     }
+  cdk_stream_tmp_set_mode (tmp, 0); /* switch to read-mode */
   cdk_stream_seek (tmp, 0);
   cdk_stream_control (tmp, CDK_STREAMCTL_RESET, 1);
   cdk_stream_set_armor_flag (tmp, 0);
