@@ -1,5 +1,5 @@
 /* write-packet.c - Write OpenPGP packets
- *  Copyright (C) 2001, 2002 Timo Schulz 
+ *        Copyright (C) 2001, 2002 Timo Schulz 
  *
  * This file is part of OpenCDK.
  *
@@ -29,7 +29,7 @@
 
 
 static int
-stream_write (CDK_STREAM s, const void *buf, size_t count)
+stream_write (CDK_STREAM s, const void * buf, size_t count)
 {
   int nwritten = cdk_stream_write (s, buf, count);
   if (nwritten == EOF)
@@ -39,7 +39,7 @@ stream_write (CDK_STREAM s, const void *buf, size_t count)
 
 
 static int
-stream_read (CDK_STREAM s, void *buf, size_t count, size_t *r_nread)
+stream_read (CDK_STREAM s, void * buf, size_t count, size_t * r_nread)
 {
   int nread = cdk_stream_read (s, buf, count);
   if (nread == EOF)
@@ -74,7 +74,7 @@ write_32 (CDK_STREAM out, u32 u)
 
 
 static int
-write_16(CDK_STREAM out, u16 u)
+write_16 (CDK_STREAM out, u16 u)
 {
   byte buf[2];
 
@@ -114,6 +114,22 @@ write_mpi (CDK_STREAM out, CDK_MPI m)
 }
 
 
+static int
+write_mpi_array (CDK_STREAM out, CDK_MPI mpi[4], int count)
+{
+  int i;
+  int rc = 0;
+  
+  for (i = 0; i < count; i++)
+    {
+      rc = write_mpi (out, mpi[i]);
+      if (rc)
+        break;
+    }
+  return rc;
+}
+
+
 static int
 pkt_encode_len (CDK_STREAM out, size_t pktlen)
 {
@@ -220,6 +236,7 @@ write_encrypted_mdc (CDK_STREAM out, cdkPKT_encrypted * enc)
 static int
 write_symkey_enc (CDK_STREAM out, cdkPKT_symkey_enc * ske)
 {
+  CDK_S2K_HD s2k;
   size_t size = 0, s2k_size = 0;
   int rc = 0;
 
@@ -229,9 +246,10 @@ write_symkey_enc (CDK_STREAM out, cdkPKT_symkey_enc * ske)
   if (DEBUG_PKT)
     _cdk_log_debug ("** write symmetric key encrypted packet\n");
 
-  if (ske->s2k.mode == 1 || ske->s2k.mode == 3)
+  s2k = ske->s2k;
+  if (s2k->mode == 1 || s2k->mode == 3)
     s2k_size = 8;
-  if (ske->s2k.mode == 3)
+  if (s2k->mode == 3)
     s2k_size++;
   size = 4 + s2k_size + ske->seskeylen;
   rc = pkt_write_head (out, size, CDK_PKT_SYMKEY_ENC);
@@ -240,14 +258,14 @@ write_symkey_enc (CDK_STREAM out, cdkPKT_symkey_enc * ske)
   if (!rc)
     rc = stream_putc (out, ske->cipher_algo);
   if (!rc)
-    rc = stream_putc (out, ske->s2k.mode);
+    rc = stream_putc (out, s2k->mode);
   if (!rc)
-    rc = stream_putc (out, ske->s2k.hash_algo);
-  if (ske->s2k.mode == 1 || ske->s2k.mode == 3)
+    rc = stream_putc (out, s2k->hash_algo);
+  if (s2k->mode == 1 || s2k->mode == 3)
     {
-      rc = stream_write (out, ske->s2k.salt, 8);
-      if (!rc && ske->s2k.mode == 3)
-	rc = stream_putc (out, ske->s2k.count);
+      rc = stream_write (out, s2k->salt, 8);
+      if (!rc && s2k->mode == 3)
+	rc = stream_putc (out, s2k->count);
     }
   return rc;
 }
@@ -256,7 +274,7 @@ write_symkey_enc (CDK_STREAM out, cdkPKT_symkey_enc * ske)
 static int
 write_pubkey_enc (CDK_STREAM out, cdkPKT_pubkey_enc * pke)
 {
-  int i, size, nenc = 0;
+  int size, nenc = 0;
   int rc = 0;
 
   if (!out || !pke)
@@ -279,12 +297,8 @@ write_pubkey_enc (CDK_STREAM out, cdkPKT_pubkey_enc * pke)
     rc = write_32 (out, pke->keyid[1]);
   if (!rc)
     rc = stream_putc (out, pke->pubkey_algo);
-  for (i = 0; i < nenc; i++)
-    {
-      rc = write_mpi (out, pke->mpi[i]);
-      if (rc)
-	break;
-    }
+  if (!rc)
+    rc = write_mpi_array (out, pke->mpi, nenc);
   return rc;
 }
 
@@ -323,9 +337,9 @@ subpkt_calc_size (CDK_SUBPKT s)
 static int
 write_signature (CDK_STREAM out, cdkPKT_signature * sig)
 {
-  byte *buf;
+  byte * buf;
   size_t nbytes;
-  int i, size, nsig = 0;
+  int size, nsig = 0;
   int rc = 0;
 
   if (!out || !sig)
@@ -404,14 +418,8 @@ write_signature (CDK_STREAM out, cdkPKT_signature * sig)
       if (!rc)
 	rc = stream_putc (out, sig->digest_start[1]);
     }
-
-  for (i = 0; i < nsig; i++)
-    {
-      rc = write_mpi (out, sig->mpi[i]);
-      if (rc)
-	break;
-    }
-
+  if (!rc)
+    rc = write_mpi_array (out, sig->mpi, nsig);
   return rc;
 }
 
@@ -420,7 +428,7 @@ static int
 write_public_key (CDK_STREAM out, cdkPKT_public_key * pk, int is_subkey)
 {
   int rc = 0;
-  int i, size = 6, pkttype;
+  int size = 6, pkttype;
   size_t npkey = 0;
 
   if (!out || !pk)
@@ -446,12 +454,8 @@ write_public_key (CDK_STREAM out, cdkPKT_public_key * pk, int is_subkey)
     rc = write_16 (out, pk->expiredate);
   if (!rc)
     rc = stream_putc (out, pk->pubkey_algo);
-  for (i = 0; i < npkey; i++)
-    {
-      rc = write_mpi (out, pk->mpi[i]);
-      if (rc)
-	break;
-    }
+  if (!rc)
+    rc = write_mpi_array (out, pk->mpi, npkey);
   return rc;
 }
 
@@ -459,9 +463,8 @@ write_public_key (CDK_STREAM out, cdkPKT_public_key * pk, int is_subkey)
 static int
 write_secret_key (CDK_STREAM out, cdkPKT_secret_key * sk)
 {
-  cdkPKT_public_key *pk = NULL;
+  cdkPKT_public_key * pk = NULL;
   size_t size = 0, npkey, nskey;
-  int i = 0;
   int rc = 0;
 
   if (!out || !sk || !sk->pk)
@@ -491,8 +494,8 @@ write_secret_key (CDK_STREAM out, cdkPKT_secret_key * sk)
     }
   if (!rc)
     rc = stream_putc (out, pk->pubkey_algo);
-  for (i = 0; i < npkey && !rc; i++)
-    rc = write_mpi (out, pk->mpi[i]);
+  if (!rc)
+    rc = write_mpi_array (out, pk->mpi, npkey);
   if (sk->is_protected == 0)
     rc = stream_putc (out, 0);
   else
@@ -503,14 +506,15 @@ write_secret_key (CDK_STREAM out, cdkPKT_secret_key * sk)
 	{
 	  rc = stream_putc (out, 0xff);
 	  if (!rc)
-	    rc = stream_putc (out, sk->protect.s2k.mode);
+	    rc = stream_putc (out, sk->protect.s2k->mode);
 	  if (!rc)
-	    rc = stream_putc (out, sk->protect.s2k.hash_algo);
-	  if (!rc && (sk->protect.s2k.mode == 1 || sk->protect.s2k.mode == 3))
+	    rc = stream_putc (out, sk->protect.s2k->hash_algo);
+	  if (!rc && (sk->protect.s2k->mode == 1
+                      || sk->protect.s2k->mode == 3))
 	    {
-	      rc = stream_write (out, sk->protect.s2k.salt, 8);
-	      if (!rc && sk->protect.s2k.mode == 3)
-		rc = stream_putc (out, sk->protect.s2k.count);
+	      rc = stream_write (out, sk->protect.s2k->salt, 8);
+	      if (!rc && sk->protect.s2k->mode == 3)
+		rc = stream_putc (out, sk->protect.s2k->count);
 	    }
 	}
       rc = stream_write (out, sk->protect.iv, sk->protect.ivlen);
@@ -522,12 +526,8 @@ write_secret_key (CDK_STREAM out, cdkPKT_secret_key * sk)
     }
   else
     {
-      for (i = 0; i < nskey; i++)
-	{
-	  rc = write_mpi (out, sk->mpi[i]);
-	  if (rc)
-	    break;
-	}
+      if (!rc)
+        rc = write_mpi_array (out, sk->mpi, nskey);
       if (!rc)
 	rc = write_16 (out, sk->csum);
     }
@@ -587,6 +587,7 @@ write_plaintext (CDK_STREAM out, cdkPKT_plaintext * pt)
           if (rc)
             break;
         }
+      wipemem (buf, sizeof buf);
     }
   return rc;
 }
@@ -600,14 +601,14 @@ write_onepass_sig (CDK_STREAM out, cdkPKT_onepass_sig * sig)
 
   if (!out || !sig)
     return CDK_Inv_Value;
+  if (sig->version != 3)
+    return CDK_Inv_Packet;
 
   if (DEBUG_PKT)
     _cdk_log_debug ("** write one pass signature packet\n");
 
-  size = 15;
+  size = 13;
   rc = pkt_write_head (out, size, CDK_PKT_ONEPASS_SIG);
-  if (!rc && sig->version != 3)
-    return CDK_Inv_Packet;
   if (!rc)
     rc = stream_putc (out, sig->version);
   if (!rc)
