@@ -33,16 +33,19 @@
 
 enum
 {
-  cArmor = 'a',
+  cSignEncr = 'b',
+  cEnarmor = 'a',
   cDearmor = 'u',
   cHelp = 'h',
   cVersion = 'V',
   cSymmetric = 'c',
   cKeyList = 'k',
+  cVerify = 'y',
   cSigList = 'l',
   cEncr = 'e',
   cDecr = 'd',
   cExport = 'x',
+  cImport = 'i',
   oCompress = 'z',
   oRecipient = 'r',
   oVerbose = 'v',
@@ -50,12 +53,16 @@ enum
   oCipherAlg = 'n',
   oDigestAlgo = 'm',
   oFetchKey = 'f',
+  cSign = 's',
+  cSignClear = 't',
+  oArmor = 'p',
+  oCompat = 'g',
 };
 
 
 static struct
 {
-  const char *name;
+  const char * name;
   int algid;
 } algo_table[] =
 {
@@ -64,18 +71,20 @@ static struct
   {"BLOWFISH", GCRY_CIPHER_BLOWFISH},
   {"TWOFISH", GCRY_CIPHER_TWOFISH},
   {"AES", GCRY_CIPHER_AES128},
+  {"AES192", GCRY_CIPHER_AES192},
+  {"AES256", GCRY_CIPHER_AES256},
   {"MD5", GCRY_MD_MD5},
   {"SHA1", GCRY_MD_SHA1},
   {"RIPEMD160", GCRY_MD_RMD160},
   {NULL},
 };
 
-char *file = "(none)";
-char *user = "(none)";
-char *outname = "(none)";
-char *cipher_algo = "(none)";
-char *digest_algo = "(none)";
-char *ks_keyid = "(none)";
+char * file = "(none)";
+char * user = "(none)";
+char * outname = "(none)";
+char * cipher_algo = "(none)";
+char * digest_algo = "(none)";
+char * ks_keyid = "(none)";
 int cipher_algid = 0;
 int digest_algid = 0;
 int use_outname = 0;
@@ -83,6 +92,14 @@ int use_outname = 0;
 
 struct poptOption arguments[] =
 {
+  {"compat", oCompat, POPT_ARG_NONE, 0, oCompat, "PGP compat mode", 0},
+  {"verify", cVerify, POPT_ARG_STRING, &file, cVerify,
+   "Verify a signature", 0},
+  {"sign", cSign, POPT_ARG_STRING, &file, cSign, "Sign a file", 0},
+  {"sign-encrypt", cSignEncr, POPT_ARG_STRING, &file, cSignEncr,
+   "Sign and encrypt a file", 0},
+  {"clearsign", cSignClear, POPT_ARG_STRING, &file, cSignClear,
+   "Sign a text file", 0},
   {"fetch-key", oFetchKey, POPT_ARG_STRING, &ks_keyid, oFetchKey,
    "Fetch key from keyserver", 0},
   {"cipher-algo", oCipherAlg, POPT_ARG_STRING, &cipher_algo, oCipherAlg,
@@ -95,13 +112,14 @@ struct poptOption arguments[] =
    "Output filename", 0},
   {"export", cExport, POPT_ARG_STRING, &user, cExport,
    "Export keys", 0},
+  {"import", cImport, POPT_ARG_STRING, &file, cImport, "Import keys", 0},
   {"list-keys", cKeyList, POPT_ARG_STRING, &user, cKeyList,
    "List keys", 0},
   {"help", cHelp, POPT_ARG_NONE, 0, cHelp, "Show help message", 0},
   {"version", cVersion, POPT_ARG_NONE, 0, cVersion,
    "Show program version", 0},
-  {"armor", cArmor, POPT_ARG_STRING, &file, cArmor,
-   "Protect a file with ASCII-Armor", 0},
+  {"enarmor", cEnarmor, POPT_ARG_STRING, &file, cEnarmor,
+   "Protect a file with ASCII-Armor", 0},  
   {"dearmor", cDearmor, POPT_ARG_STRING, &file, cDearmor,
    "Remove ASCII-Armor from a file", 0},
   {"compress", oCompress, POPT_ARG_NONE, 0, oCompress,
@@ -113,6 +131,7 @@ struct poptOption arguments[] =
   {"encrypt", cEncr, POPT_ARG_STRING, &file, cEncr, "Encrypt a file", 0},
   {"decrypt", cDecr, POPT_ARG_STRING, &file, cDecr, "Decrypt a file", 0},
   {"verbose", oVerbose, POPT_ARG_NONE, 0, oVerbose, "Verbosity", 0},
+  {"armor", oArmor, POPT_ARG_NONE, 0, oArmor, "Armor output", 0},
   {NULL}
 };
 
@@ -125,10 +144,17 @@ out_of_core (void)
 }
 
 
-static char *
-do_add_ext (const char *filename, const char *ext)
+static void
+callback (void * opaque, int type, const char * s)
 {
-  char *output = NULL;
+  fprintf (stdout, "type=%d val=%s\n", type, s);
+}
+
+
+static char *
+do_add_ext (const char * filename, const char * ext)
+{
+  char * output = NULL;
 
   if (!filename)
     return NULL;
@@ -142,9 +168,9 @@ do_add_ext (const char *filename, const char *ext)
 
 
 static char *
-do_remove_ext (const char *filename)
+do_remove_ext (const char * filename)
 {
-  char *output = NULL;
+  char * output = NULL;
 
   if (!filename)
     return NULL;
@@ -186,7 +212,7 @@ static const char *
 key_get_date (long timestamp)
 {
   static char kdate[32];
-  struct tm *tm;
+  struct tm * tm;
 
   tm = localtime (&timestamp);
   snprintf (kdate, sizeof kdate - 1, "%04d-%02d-%02d",
@@ -246,14 +272,14 @@ static void
 do_show_key (CDK_KBNODE pk, int with_sigs)
 {
   CDK_KBNODE p, ctx = NULL;
-  CDK_PACKET *pkt;
+  CDK_PACKET * pkt;
   
   while ((p = cdk_kbnode_walk (pk, &ctx, 0)))
     {
       pkt = cdk_kbnode_get_packet (p);
       if (pkt->pkttype == CDK_PKT_PUBLIC_KEY)
 	{
-	  cdkPKT_public_key *key = pkt->pkt.public_key;
+	  cdkPKT_public_key * key = pkt->pkt.public_key;
 	  fprintf (stdout, "pub  %04d%c/%08lX %s\n",
 		   cdk_pk_get_nbits (key),
 		   key_get_letter (key->pubkey_algo),
@@ -262,7 +288,7 @@ do_show_key (CDK_KBNODE pk, int with_sigs)
 	}
       else if (pkt->pkttype == CDK_PKT_PUBLIC_SUBKEY)
 	{
-	  cdkPKT_public_key *key = pkt->pkt.public_key;
+	  cdkPKT_public_key * key = pkt->pkt.public_key;
 	  fprintf (stdout, "sub  %04d%c/%08lX %s\n",
 		   cdk_pk_get_nbits (key),
 		   key_get_letter (key->pubkey_algo),
@@ -271,13 +297,15 @@ do_show_key (CDK_KBNODE pk, int with_sigs)
 	}
       else if (pkt->pkttype == CDK_PKT_USER_ID)
 	{
-	  cdkPKT_user_id *id = pkt->pkt.user_id;
-	  fprintf (stdout, "uid  %s  %s\n",
-		   id->is_revoked ? "[revoked]" : "         ", id->name);
+	  cdkPKT_user_id * id = pkt->pkt.user_id;
+          char * uid = cdk_utf8_decode (id->name, id->len, 0);
+          fprintf (stdout, "uid %s %s\n",
+                   id->is_revoked ? "[revoked]" : "         ", uid);
+          cdk_free (uid);
 	}
       else if (with_sigs && pkt->pkttype == CDK_PKT_SIGNATURE)
 	{
-	  cdkPKT_signature *sig = pkt->pkt.signature;
+	  cdkPKT_signature * sig = pkt->pkt.signature;
 	  fprintf (stdout, " sig%c %c %08lX %s\n",
 		   sig_get_letter (sig),
 		   sig_get_status (sig),
@@ -292,11 +320,13 @@ int
 do_list_keys (CDK_STRLIST keys)
 {
   const char * ring, * s;
+  unsigned char fpr[20];
+  CDK_PACKET * pkt;
   CDK_STREAM inp;
   CDK_KBNODE pk = NULL;
   CDK_KEYDB_HD khd = NULL;
   CDK_STRLIST ctx = NULL;
-  int rc = 0;
+  int rc = 0, i;
 
   if (keys)
     {
@@ -307,7 +337,16 @@ do_list_keys (CDK_STRLIST keys)
 	  if (rc)
 	    fprintf (stdout, "get key `%s': %s\n", s, cdk_strerror (rc));
 	  if (!rc)
-	    do_show_key (pk, 0);
+            {
+              do_show_key (pk, 0);
+              pkt = cdk_kbnode_get_packet (pk);
+              printf ("Fingerprint ");
+              cdk_pk_get_fingerprint (pkt->pkt.public_key, fpr);
+              for (i = 0; i < 20/2; i++)
+                printf ("%02X%02X ", fpr[2*i], fpr[2*i+1]);
+              printf ("\n");
+            }
+              
 	  cdk_kbnode_release (pk);
 	  pk = NULL;
 	}
@@ -326,11 +365,16 @@ do_list_keys (CDK_STRLIST keys)
       fprintf (stderr, "`%s': %s\n", ring, cdk_strerror (rc));
       return rc;
     }
-  while (!(rc = cdk_keydb_get_keyblock (inp, &pk) && !cdk_stream_eof (inp)))
+
+  while (!(rc = cdk_keydb_get_keyblock (inp, &pk)))
     {
       do_show_key (pk, 0);
+      fputc ('\n', stdout);
       cdk_kbnode_release (pk);
       pk = NULL;
+      rc = cdk_stream_eof (inp);
+      if (rc)
+        break;
     }
   cdk_stream_close (inp);
   return 0;
@@ -366,13 +410,65 @@ do_export_keys (CDK_STRLIST keys, int secret)
 }
 
 
+int
+do_import_keys (const char * keyfile)
+{
+  CDK_KEYDB_HD hd;
+  CDK_KBNODE node = NULL, n, ctx = NULL;
+  CDK_STREAM s;
+  int rc, res[4];
+
+  hd = cdk_keydb_get_ctx (0, -1);
+  if (!hd)
+    {
+      fprintf (stderr, "No public keyring available.\n");
+      return -1;
+    }
+  rc = cdk_stream_open (keyfile, &s);
+  if (!s)
+    {
+      fprintf (stderr, "open `%s' failed: %s\n", keyfile, cdk_strerror (rc));
+      return -1;
+    }
+  rc = cdk_keydb_get_keyblock (s, &node);
+  if (!node)
+    {
+      cdk_stream_close (s);
+      fprintf (stderr, "get keyblock failed: %s\n", cdk_strerror (rc));
+      return -1;
+    }
+  rc = cdk_keydb_import (hd, node, res);
+  if (rc)
+    fprintf (stderr, "import failed: %s\n", cdk_strerror (rc));
+  else
+    {
+      while ((n = cdk_kbnode_walk (node, &ctx, 0)))
+        {
+          CDK_PACKET * pkt = cdk_kbnode_get_packet (n);
+          if (pkt->pkttype == CDK_PKT_USER_ID)
+            {
+              printf ("public keys read/updated: %d/%d\n"
+                      "secret keys read/updated: %d/%d\n",
+                      res[0], res[2], res[1], res[3]);
+              printf ("`%s' successfully imported.\n",
+                      pkt->pkt.user_id->name);
+              break;
+            }
+        }
+    }
+  cdk_stream_close (s);
+  cdk_kbnode_release (node);
+  return 0;
+}
+  
+
 int
 do_list_sigs (CDK_STRLIST keys)
 {
   CDK_KEYDB_HD hd = NULL;
   CDK_KBNODE pk = NULL;
   CDK_STRLIST ctx = NULL;
-  const char *s;
+  const char * s;
   int kstat = 0;
   int rc = 0;
 
@@ -398,7 +494,7 @@ do_list_sigs (CDK_STRLIST keys)
 char *
 get_passphrase (const char *prompt)
 {
-  char *p = NULL;
+  char * p = NULL;
   int maxlen = 255;
 
   fprintf (stdout, "%s", prompt);
@@ -449,21 +545,90 @@ ks_fetch_key (const char * kid)
 }
 
 
+static const char *
+get_sig_status (int id)
+{
+  const char * s = "unknown error";
+
+  switch (id)
+    {
+    case CDK_SIGSTAT_NONE : s = "no signature"; break;
+    case CDK_SIGSTAT_GOOD : s = "good signature"; break;
+    case CDK_SIGSTAT_BAD  : s = "bad signature"; break;
+    case CDK_SIGSTAT_NOKEY: s = "no key for checking the signature"; break; 
+    }
+  return s;
+}
+
+static const char *
+get_sig_algo (int pubkey_algo)
+{
+  switch (pubkey_algo)
+    {
+    case GCRY_PK_RSA_E:
+    case GCRY_PK_RSA_S:
+    case GCRY_PK_RSA  : return "RSA";
+    case GCRY_PK_ELG  :
+    case GCRY_PK_ELG_E: return "ELG";
+    case GCRY_PK_DSA  : return "DSA";
+    default           : return "???"; 
+    }
+  return NULL; 
+}
+
+static const char *
+get_sig_time (unsigned long timestamp)
+{
+  static char buf[128];
+  struct tm * tmbuf;
+
+  tmbuf = localtime (&timestamp);
+  snprintf (buf, sizeof buf-1, "%04d-%02d-%02d",
+            tmbuf->tm_year+1900, tmbuf->tm_mon+1, tmbuf->tm_mday);
+  return buf;  
+}
+
+
+void
+print_verify_stats (CDK_HD hd)
+{
+  const unsigned long * keyid;
+  unsigned long created;
+  int sigstat, pkalgo;
+
+  sigstat = cdk_sig_get_ulong_attr (hd, 0, CDK_ATTR_STATUS);
+  if (sigstat)
+    {
+      keyid = cdk_sig_get_data_attr (hd, 0, CDK_ATTR_KEYID);
+      created = cdk_sig_get_ulong_attr (hd, 0, CDK_ATTR_CREATED);
+      pkalgo = cdk_sig_get_ulong_attr (hd, 0, CDK_ATTR_ALGO_PK);
+      printf ("(%s) keyid %08lX%08lX algo %s created %s\n",
+              get_sig_status (sigstat), keyid[0], keyid[1],
+              get_sig_algo (pkalgo), get_sig_time (created));
+    }
+}
+
+
 int
-main (int argc, char **argv)
+main (int argc, char ** argv)
 {
   CDK_HD hd;
   CDK_STRLIST remusr = NULL, klist = NULL;
   poptContext opt_ctx;
-  char *output = NULL;
+  char * output = NULL;
   char u[128];
   int c = 0, verbose = 0, i = 0;
   int rc = 0;
 
-  cdk_keydb_add_resource ("/home/twoaday/.gnupg/pubring.gpg", 0);
-  cdk_passphrase_cb_set (get_passphrase);
+  cdk_keydb_add_resource ("/home/twoaday/.gnupg/pubring.cdk", 0);
+  /*cdk_keydb_add_resource ("pubring.gpg", 0);*/
+  cdk_keydb_add_resource ("sec-with-pwd.gpg", 1);
+  /*cdk_keydb_add_resource ("secring.gpg", 1);*/
+  /*cdk_keydb_add_resource ("sec2.gpg", 1);*/
+  /*cdk_passphrase_cb_set (get_passphrase);*/
 
   cdk_handle_new (&hd);
+  cdk_handle_set_callback (hd, callback, NULL);
   opt_ctx = poptGetContext ("pred", argc, argv, arguments, 0);
   while ((c = poptGetNextOpt (opt_ctx)) >= 0)
     {
@@ -473,12 +638,20 @@ main (int argc, char **argv)
 	  verbose++;
 	  break;
 
+        case 'p':
+          cdk_handle_set_armor (hd, 1);
+          break;
+
 	case 'o':
 	  if (verbose)
 	    fprintf (stdout, "Output filename `%s'\n", outname);
 	  use_outname = 1;
 	  break;
 
+        case 'g':
+          cdk_handle_set_compat (hd, 1);
+          break;
+
 	case 'h':
 	  fprintf (stderr, "opencdk (MinPG) %s\n\n", VERSION);
 	  poptPrintHelp (opt_ctx, stdout, 0);
@@ -507,6 +680,15 @@ main (int argc, char **argv)
 		     cipher_algo, cipher_algid);
 	  break;
 
+        case 'y':
+          rc = cdk_file_verify_clearsign (hd, file, NULL);
+          if (rc)
+            fprintf (stderr, "verify `%s' failed: %s\n", file,
+                     cdk_strerror (rc));
+          else
+            print_verify_stats (hd);
+          break;
+          
 	case 'u':
 	  output = cdk_calloc (1, strlen (file));
 	  if (strstr (file, ".asc"))
@@ -567,6 +749,10 @@ main (int argc, char **argv)
 	  do_export_keys (klist, 0);
 	  break;
 
+        case 'i':
+          do_import_keys (file);
+          break;
+
         case 'f':
           ks_fetch_key (ks_keyid);
           break;
@@ -583,6 +769,7 @@ main (int argc, char **argv)
 	      fgets (u, sizeof u, stdin);
 	      cdk_strlist_add (&remusr, user);
 	    }
+          cdk_set_log_level (CDK_LOG_DEBUG);
 	  output = do_add_ext (file, ".gpg");
 	  if (verbose)
 	    fprintf (stdout, "encrypt `%s' -> %s\n", file, output);
@@ -593,19 +780,63 @@ main (int argc, char **argv)
 	  break;
 
 	case 'd':
+          cdk_set_log_level (CDK_LOG_DEBUG);
 	  output = do_remove_ext (file);
 	  if (verbose)
 	    fprintf (stdout, "decrypt `%s' -> %s\n", file, output);
 	  rc = cdk_file_decrypt (hd, file, output);
 	  if (rc)
-	    fprintf (stdout, "minpg: decrypt file `%s': %s\n",
+	    fprintf (stdout, "minpg: decrypt file `%s' failed: %s\n",
 		     file, cdk_strerror (rc));
 	  break;
+
+        case 's':
+          if (!remusr)
+            {
+              fprintf (stdout, "No user was chosen.\n");
+              exit (0);
+            } 
+          output = do_add_ext (file, ".sig");
+          if (verbose)
+            fprintf (stdout, "sign `%s' -> %s\n", file, output);
+          rc = cdk_file_sign (hd, remusr, NULL, file, output, 0, 0);
+          if (rc)
+            fprintf (stderr, "minpg: sign file `%s' failed: %s\n",
+                     file, cdk_strerror (rc));
+          break;
+
+        case 't':
+          if (!remusr)
+            {
+              fprintf (stdout, "No user was chosen.\n");
+              exit (0);
+            }
+          output = do_add_ext (file, ".asc");
+          rc = cdk_file_clearsign (hd, remusr, file, output);
+          if (rc)
+            fprintf (stderr, "minpg: clearsign file `%s': failed: %s\n",
+                     file, cdk_strerror (rc));
+          break;
+
+        case 'b':
+          if (!remusr)
+            {
+              fprintf (stderr, "No recipient was chosen.\n");
+              exit (0);
+            }
+          output = do_add_ext (file, ".gpg");
+          cdk_strlist_add (&klist, "opencdk");
+          rc = cdk_file_sign (hd, klist, remusr, file, output, 0, 1);
+          if (rc)
+            fprintf (stderr, "minpg: sign and encrypt `%s' failed: %s\n",
+                     file, cdk_strerror (rc));
+          break;
 	}
     }
 
   cdk_handle_free (hd);
   cdk_strlist_free (remusr);
+  cdk_free (output);
   return 0;
 }
 #endif /* HAVE_LIBPOPT */
